#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : im_chat_qa.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : QA 问答记录表

import logging

from django_base_ai.system.models import IMChatQA
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class IMChatQASerializer(CustomModelSerializer):
    class Meta:
        model = IMChatQA
        fields = "__all__"
        read_only_fields = ["id"]


class IMChatQACreateUpdateSerializer(CustomModelSerializer):
    class Meta:
        model = IMChatQA
        fields = ["is_it_solved"]
        read_only_fields = ["id", "ai_desk_app_manage"]


class IMChatQAViewSet(CustomModelViewSet):
    """
    QA问答表
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = IMChatQA.objects.all()
    serializer_class = IMChatQASerializer
    create_serializer_class = IMChatQACreateUpdateSerializer
    update_serializer_class = IMChatQACreateUpdateSerializer
    # extra_filter_backends = []
    filter_fields = ["ai_desk_app_manage"]



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : apps.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 应用与消息任务管理接口
import json
import logging
import uuid
from datetime import datetime

from django.core.cache import cache
from django.shortcuts import HttpResponse
from rest_framework.decorators import action
from rest_framework.permissions import AllowAny, IsAdminUser

from django_base_ai.system.models import (
    Apps,
    MessageCenter,
    MessageCenterTargetUser,
    OperationLog,
    SendMessageTask,
    Users,
)
from django_base_ai.utils import xss_cleaned
from django_base_ai.utils.json_response import DetailResponse, ErrorResponse, SuccessResponse
from django_base_ai.utils.message import client, smtp_email
from django_base_ai.utils.permission import OpenApiPermission
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class AppsSerializer(CustomModelSerializer):
    class Meta:
        model = Apps
        exclude = [
            "secret",
        ]
        read_only_fields = ["id"]


class AppsCreateSerializer(CustomModelSerializer):
    """
    地区管理 创建/更新时的列化器
    """

    def save(self, **kwargs):
        data = super().save(**kwargs)
        data.agent_id = data.agent_id if data.agent_id else f"agent_{uuid.uuid4()}"
        data.secret = data.secret if data.secret else f"secret_{uuid.uuid1() + uuid.uuid4()}"
        data.creator = self.request.user.id
        data.modifier = self.request.user.id
        data.dept_belong_id = self.request.user.dept.id if self.request.user and self.request.user.dept else 1
        data.save()
        return data

    class Meta:
        model = Apps
        exclude = ["secret"]
        read_only_fields = ["id"]


class AppsUpdateSerializer(CustomModelSerializer):
    """
    地区管理 创建/更新时的列化器
    """

    class Meta:
        model = Apps
        exclude = ["secret"]
        read_only_fields = ["id"]


class AppsViewSet(CustomModelViewSet):
    """
    应用管理接口
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = Apps.objects.all()
    serializer_class = AppsSerializer
    create_serializer_class = AppsCreateSerializer
    update_serializer_class = AppsUpdateSerializer
    search_fields = ["title"]

    # extra_filter_backends = []

    @action(methods=["get"], detail=False, permission_classes=[IsAdminUser])
    def agent_info(self, request, *args, **kwargs):
        """
        查看应用密钥
        :param request:
        :return:
        """
        nid = request.GET.get("nid", 0)
        if nid == 0:
            return ErrorResponse(msg="参数错误")
        app_info = Apps.objects.filter(id=nid, creator=request.user.id).first()
        if app_info is None:
            return ErrorResponse(msg="您无权访问或应用不存在")
        return DetailResponse({"agent_id": app_info.agent_id, "secret": app_info.secret})

    @action(methods=["post"], detail=False, permission_classes=[AllowAny], authentication_classes=[])
    def get_suite_token(self, request, *args, **kwargs):
        """
        第三方应用登录
        :param request:
        :return:
        """
        agent_id = str(request.data.get("agent_id", "")).strip()
        secret = str(request.data.get("secret", "")).strip()
        if agent_id == "" or secret == "":
            return ErrorResponse(msg="参数错误,应用id或密钥不能为空")
        app_info = Apps.objects.filter(agent_id=agent_id).first()
        if app_info is None or secret != app_info.secret:
            return ErrorResponse(msg="参数错误,请检查参数是否合法")
        ticket = f"{uuid.uuid1().hex + uuid.uuid4().hex}{int(datetime.now().timestamp())}"
        cache.set(ticket, app_info.id, timeout=1800)
        return DetailResponse(data=ticket)

    @action(methods=["post"], detail=False, permission_classes=[OpenApiPermission], authentication_classes=[])
    def async_send_email(self, request, *args, **kwargs):
        """
        异步发送邮件
        :param request:
        :param args:
        :param kwargs:
        {
        "subject":"测试",
        "content":"邮件内容",
        "to_mails":["2262511449@qq.com"],
        "file_list":[]
        }
        :return:
        """
        from celery_tasks.tasks import send_email_message

        apps_id = cache.get(request.META.get("HTTP_AUTHORIZATION", None))
        apps_info = Apps.objects.filter(id=apps_id).first()
        # 邮件标题
        subject = request.data.get("subject", None)
        # 邮件内容
        content = request.data.get("content", None)
        # 收件人
        to_mails = request.data.get("to_mails", [])
        # 附件
        file_list = request.data.get("file_list", [])
        if not subject or not content or not to_mails:
            return ErrorResponse(msg=f"参数错误,邮件标题:{subject},收件人:{to_mails},邮件内容不能为空")
        app_user = Users.objects.filter(id=apps_info.creator).first()
        send_message_task = SendMessageTask.objects.create(
            apps_id=apps_id, creator=apps_info.creator, modifier=apps_info.modifier
        )
        result = send_email_message.apply_async(
            args=[
                send_message_task.id,
                subject,
                content,
                str(apps_info.creator),
                app_user.dept.id if app_user.dept else "",
                to_mails,
                file_list,
            ]
        )
        send_message_task.task_name = result.status
        send_message_task.task_id = result.task_id
        send_message_task.save()
        return DetailResponse(msg="后台任务已启动,请稍后刷新当前页面,重新加载数据", data=result.task_id)

    @action(methods=["post"], detail=False, permission_classes=[OpenApiPermission], authentication_classes=[])
    def sync_send_email(self, request, *args, **kwargs):
        """
        同步发送邮件
        :param request:
        :param args:
        :param kwargs:
        {
        "subject":"测试",
        "content":"邮件内容",
        "to_mails":["2262511449@qq.com"],
        "file_list":[]
        }
        :return:
        """
        apps_id = cache.get(request.META.get("HTTP_AUTHORIZATION", None))
        # 邮件标题
        subject = xss_cleaned.cleaned_string(request.data.get("subject", ""))
        # 简要描述
        short_content = request.data.get("short_content", "")
        # 邮件内容
        content = request.data.get("content", "")
        # 收件人
        to_mails = list(set(request.data.get("to_mails", [])))
        # 附件
        file_list = request.data.get("file_list", [])
        if not subject or not content or not to_mails:
            return ErrorResponse(msg=f"参数错误,邮件标题:{subject},收件人:{to_mails},邮件内容不能为空")
        apps_info = Apps.objects.filter(id=apps_id).first()
        smtp_email_obj = smtp_email.SmtpEmail()
        user_info = Users.objects.filter(email__in=to_mails).values("id", "email", "dept_id")
        user_info_dict = {
            item.get("email"): {"id": item.get("id"), "dept_id": item.get("dept_id")} for item in user_info
        }
        for to_mail in to_mails:
            send_user_dict = user_info_dict.get(to_mail, 0)
            if send_user_dict:
                user_id = send_user_dict.get("id", None)
                send_mails_result = smtp_email_obj.send(to_mail, subject, content, file_list)
                message_center = MessageCenter.objects.create(
                    title=subject,
                    short_content=short_content,
                    content=content,
                    message_type=1,
                    file_list=file_list,
                    ext_kwargs={
                        "msg_type": 1,
                        "user_info": {"uid": 2, "name": apps_info.title, "username": None, "avatar": apps_info.icon},
                    },
                    dept_belong_id=send_user_dict.get("dept_id", ""),
                    creator=user_id,
                    modifier=user_id,
                )
                MessageCenterTargetUser.objects.create(
                    users_id=user_id,
                    messagecenter=message_center,
                    send_mails_result=send_mails_result,
                    dept_belong_id=send_user_dict.get("dept_id", ""),
                    creator=user_id,
                    modifier=user_id,
                )
        return DetailResponse(msg="发送成功", data=[])

    @action(methods=["post"], detail=False, permission_classes=[OpenApiPermission], authentication_classes=[])
    def send_client_msg(self, request, *args, **kwargs):
        """
        发送客户端通知
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        apps_id = cache.get(request.META.get("HTTP_AUTHORIZATION", None))
        apps_info = Apps.objects.filter(id=apps_id).first()
        app_user = Users.objects.filter(id=apps_info.creator).first()
        SendMessageTask.objects.create(
            apps_id=apps_id,
            send_type=1,
            dept_belong_id=app_user.dept.id if app_user.dept else "",
            creator=apps_info.creator,
            modifier=apps_info.modifier,
        )
        title = request.data.get("title", "")
        short_content = request.data.get("short_content", "")
        message_type = request.data.get("message_type", "")
        thumbnail = request.data.get("thumbnail", "")
        content = xss_cleaned.cleaned_string(request.data.get("content", ""))
        url = request.data.get("url", "")
        send_user_info = request.data.get("send_user_info", "")
        if not title or not short_content or not content or not url or not send_user_info:
            return ErrorResponse(
                msg=f"参数错误,请检查参数是否合法,title={title},short_content={short_content},message_type={message_type},content={content},url={url},send_user_info={send_user_info}"
            )
        client_obj = client.Client()
        payload = {
            "title": title,
            "short_content": short_content,
            "message_type": message_type,
            "thumbnail": thumbnail,
            "content": content,
            "url": url,
            "send_user_info": send_user_info,
        }
        send_json = client_obj.send(payload)
        print(send_json)
        return HttpResponse(json.dumps(send_json), content_type="application/json")

    @action(methods=["post"], detail=False, permission_classes=[OpenApiPermission], authentication_classes=[])
    def cleaned_xss(self, request, *args, **kwargs):
        """
        XSS 注入
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        safe_content = xss_cleaned.cleaned_string(request.data.get("content"))
        return SuccessResponse(data=safe_content)

    @action(methods=["post"], detail=False, permission_classes=[OpenApiPermission], authentication_classes=[])
    def access_log(self, request, *args, **kwargs):
        """
        操作记录
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        apps_id = cache.get(request.META.get("HTTP_AUTHORIZATION", None))
        apps_info = Apps.objects.filter(id=apps_id).first()
        username = request.data.get("um", "")
        current_user = Users.objects.filter(username=username).first()
        datainfo = {
            "appid": apps_info.id,
            "request_modular": request.data.get("request_modular", ""),
            "request_path": request.data.get("request_path", ""),
            "request_body": request.data.get("request_body", ""),
            "request_method": request.data.get("request_method", ""),
            "request_msg": request.data.get("request_msg", ""),
            "request_ip": request.data.get("request_ip", ""),
            "request_browser": request.data.get("request_browser", ""),
            "response_code": request.data.get("response_code", ""),
            "request_os": request.data.get("request_os", ""),
            "json_result": request.data.get("json_result", {}),
            "status": request.data.get("status", False),
            "description": request.data.get("description", ""),
            "dept_belong_id": current_user.dept.id if current_user and current_user.dept else "",
            "creator": current_user.creator if current_user else "",
            "modifier": current_user.modifier if current_user else "",
        }
        OperationLog.objects.create(**datainfo)
        return DetailResponse(msg="操作成功")



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : ueditor_settings.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : UEditor 富文本工具栏与上传配置，供前端与 file_list 等使用。

import logging

from django.conf import settings as g_settings  # 全局设置

logger = logging.getLogger(__name__)

# 工具栏样式，可以添加任意多的模式
toolbars_settings = {
    "besttome": [
        [
            "source",
            "undo",
            "redo",
            "bold",
            "italic",
            "underline",
            "forecolor",
            "backcolor",
            "superscript",
            "subscript",
            "justifyleft",
            "justifycenter",
            "justifyright",
            "insertorderedlist",
            "insertunorderedlist",
            "blockquote",
            "formatmatch",
            "removeformat",
            "autotypeset",
            "inserttable",
            "pasteplain",
            "wordimage",
            "searchreplace",
            "map",
            "preview",
            "fullscreen",
        ],
        [
            "insertcode",
            "paragraph",
            "fontfamily",
            "fontsize",
            "link",
            "unlink",
            "insertimage",
            "insertvideo",
            "attachment",
            "emotion",
            "date",
            "time",
        ],
    ],
    "mini": [
        [
            "source",
            "|",
            "undo",
            "redo",
            "|",
            "bold",
            "italic",
            "underline",
            "formatmatch",
            "autotypeset",
            "|",
            "forecolor",
            "backcolor",
            "|",
            "link",
            "unlink",
            "|",
            "simpleupload",
            "attachment",
        ]
    ],
    "normal": [
        [
            "source",
            "|",
            "undo",
            "redo",
            "|",
            "bold",
            "italic",
            "underline",
            "removeformat",
            "formatmatch",
            "autotypeset",
            "|",
            "forecolor",
            "backcolor",
            "|",
            "link",
            "unlink",
            "|",
            "simpleupload",
            "emotion",
            "attachment",
            "|",
            "inserttable",
            "deletetable",
            "insertparagraphbeforetable",
            "insertrow",
            "deleterow",
            "insertcol",
            "deletecol",
            "mergecells",
            "mergeright",
            "mergedown",
            "splittocells",
            "splittorows",
            "splittocols",
        ]
    ],
}

# 默认的Ueditor设置，请参见ueditor.config.js
ueditor_settings = {
    "toolbars": toolbars_settings["normal"],
    "autoFloatEnabled": False,
    "defaultPathFormat": "%(basename)s_%(datetime)s_%(rnd)s.%(extname)s",  # 默认保存上传文件的命名方式
}
# 请参阅php文件夹里面的config.json进行配置
ueditor_upload_settings = {
    # 上传图片配置项
    "imageActionName": "uploadimage",  # 执行上传图片的action名称
    "imageMaxSize": 10485760,  # 上传大小限制，单位B,10M
    "imageFieldName": "upfile",  # * 提交的图片表单名称 */
    "imagePathFormat": "",
    "imageInsertAlign": "none",
    "imageAllowFiles": [".png", ".jpg", ".jpeg", ".gif", ".bmp"],  # 上传图片格式显示
    # 涂鸦图片上传配置项 */
    "scrawlActionName": "uploadscrawl",  # 执行上传涂鸦的action名称 */
    "scrawlFieldName": "upfile",  # 提交的图片表单名称 */
    "scrawlMaxSize": 10485760,  # 上传大小限制，单位B  10M
    "scrawlPathFormat": "",
    "scrawlInsertAlign": "none",
    # 截图工具上传 */
    "snapscreenActionName": "uploadimage",  # 执行上传截图的action名称 */
    "snapscreenPathFormat": "",
    "snapscreenInsertAlign": "none",  # /* 插入的图片浮动方式 */
    # 抓取远程图片配置 */
    "catcherLocalDomain": ["127.0.0.1", "localhost", "img.baidu.com"],
    "catcherPathFormat": "",
    "catcherActionName": "catchimage",  # 执行抓取远程图片的action名称 */
    "catcherFieldName": "source",  # 提交的图片列表表单名称 */
    "catcherMaxSize": 10485760,  # 上传大小限制，单位B */
    "catcherAllowFiles": [".png", ".jpg", ".jpeg", ".gif", ".bmp"],  # 抓取图片格式显示 */
    "catcherInsertAlign": "none",  # /* 插入的图片浮动方式 */
    # 上传视频配置 */
    "videoActionName": "uploadvideo",  # 执行上传视频的action名称 */
    "videoPathFormat": "",
    "videoFieldName": "upfile",  # 提交的视频表单名称 */
    "videoMaxSize": 102400000,  # 上传大小限制，单位B，默认100MB */
    "videoAllowFiles": [
        ".flv",
        ".swf",
        ".mkv",
        ".avi",
        ".rm",
        ".rmvb",
        ".mpeg",
        ".mpg",
        ".ogg",
        ".ogv",
        ".mov",
        ".wmv",
        ".mp4",
        ".webm",
        ".mp3",
        ".wav",
        ".mid",
    ],  # 上传视频格式显示 */
    # 上传文件配置 */
    "fileActionName": "uploadfile",  # controller里,执行上传视频的action名称 */
    "filePathFormat": "",
    "fileFieldName": "upfile",  # 提交的文件表单名称 */
    "fileMaxSize": 204800000,  # 上传大小限制，单位B，200MB */
    "fileAllowFiles": [
        ".png",
        ".jpg",
        ".jpeg",
        ".gif",
        ".bmp",
        ".flv",
        ".swf",
        ".mkv",
        ".avi",
        ".rm",
        ".rmvb",
        ".mpeg",
        ".mpg",
        ".ogg",
        ".ogv",
        ".mov",
        ".wmv",
        ".mp4",
        ".webm",
        ".mp3",
        ".wav",
        ".mid",
        ".rar",
        ".zip",
        ".tar",
        ".gz",
        ".7z",
        ".bz2",
        ".cab",
        ".iso",
        ".doc",
        ".docx",
        ".xls",
        ".xlsx",
        ".ppt",
        ".pptx",
        ".pdf",
        ".txt",
        ".md",
        ".xml",
    ],  # 上传文件格式显示 */
    # 列出指定目录下的图片 */
    "imageManagerActionName": "listimage",  # 执行图片管理的action名称 */
    "imageManagerListPath": "",
    "imageManagerListSize": 30,  # 每次列出文件数量 */
    "imageManagerAllowFiles": [".png", ".jpg", ".jpeg", ".gif", ".bmp"],  # 列出的文件类型 */
    # 列出指定目录下的文件 */
    "fileManagerActionName": "listfile",  # 执行文件管理的action名称 */
    "fileManagerListPath": "",
    "fileManagerListSize": 30,  # 每次列出文件数量 */
    "fileManagerAllowFiles": [
        ".png",
        ".jpg",
        ".jpeg",
        ".gif",
        ".bmp",
        ".tif",
        ".psd.flv",
        ".swf",
        ".mkv",
        ".avi",
        ".rm",
        ".rmvb",
        ".mpeg",
        ".mpg",
        ".ogg",
        ".ogv",
        ".mov",
        ".wmv",
        ".mp4",
        ".webm",
        ".mp3",
        ".wav",
        ".mid",
        ".rar",
        ".zip",
        ".tar",
        ".gz",
        ".7z",
        ".bz2",
        ".cab",
        ".iso",
        ".doc",
        ".docx",
        ".xls",
        ".xlsx",
        ".ppt",
        ".pptx",
        ".pdf",
        ".txt",
        ".md",
        ".xml",
        ".exe",
        ".com",
        ".dll",
        ".msi",
    ],  # 列出的文件类型 */
}


# 更新配置：从用户配置文件settings.py重新读入配置UEDITOR_SETTINGS,覆盖默认
def update_user_settings():
    user_settings = getattr(g_settings, "UEDITOR_SETTINGS", {}).copy()
    if "config" in user_settings:
        ueditor_settings.update(user_settings["config"])
    if "upload" in user_settings:
        ueditor_upload_settings.update(user_settings["upload"])


# 读取用户Settings文件并覆盖默认配置
update_user_settings()


# 取得配置项参数
def get_ueditor_settings(key, default=None):
    return ueditor_settings.get(key, default)



#!/usr/bin/env python
"""
@Project ：django_base_ai
@File    ：scan_qr_cv.py
@Author  ：congxing.wang
@Date    ：2026/02/25
@Desc    ：二维码与验证码识别（OpenCV）
"""


class ScanQR:
    def __init__(self):
        pass

    def info(self, file_path):
        import cv2

        detector = cv2.wechat_qrcode_WeChatQRCode(
            "./opencv_3rdparty/detect.prototxt",
            "./opencv_3rdparty/detect.caffemodel",
            "./opencv_3rdparty/sr.prototxt",
            "./opencv_3rdparty/sr.caffemodel",
        )
        img = cv2.imread(file_path)
        res, points = detector.detectAndDecode(img)
        return res, points


if __name__ == "__main__":
    scan_qr = ScanQR()
    print(scan_qr.info("./opencv_3rdparty/img.jpg"))



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : convert_pdf.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 根据 URL 生成 PDF 接口
# import pdfkit
import logging

from rest_framework.views import APIView

from django_base_ai.utils.json_response import DetailResponse

logger = logging.getLogger(__name__)


class ConvertPdfView(APIView):
    authentication_classes = []
    permission_classes = []

    def get(self, request):
        # pdfkit.from_url('https://www.baidu.com/', 'out.pdf')
        return DetailResponse(msg="成功")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
system 应用信号：MessageCenter 创建时生成目标用户记录；Users 创建/更新/删除时回调应用；Tags 同。
含防止 AnonymousUser 写入库的 pre_save/post_save 处理。
"""
import json
import logging

import httpx
from django.db.models import Q
from django.db.models.signals import post_delete, post_save, pre_save
from django.dispatch import receiver

from django_base_ai.system.models import Apps, MessageCenter, MessageCenterTargetUser, Tags, Users
from django_base_ai.system.views.tags import TagsSerializer
from django_base_ai.system.views.user import UserSerializer
from django_base_ai.utils.format_core_base_auth import get_auth_user_info

logger = logging.getLogger(__name__)

"""
from django.core.signals import request_finished
from django.core.signals import request_started
from django.core.signals import got_request_exception
from django.db.models.signals import class_prepared
from django.db.models.signals import pre_init, post_init
from django.db.models.signals import pre_save, post_save
from django.db.models.signals import pre_delete, post_delete
from django.db.models.signals import m2m_changed
from django.db.models.signals import pre_migrate, post_migrate
from django.test.signals import setting_changed
from django.test.signals import template_rendered
from django.db.backends.signals import connection_created
"""
"""
Model signals
    pre_init                    # django的modal执行其构造方法前，自动触发
    post_init                   # django的modal执行其构造方法后，自动触发
    pre_save                    # django的modal对象保存save()前，自动触发
    post_save                   # django的modal对象保存save()后，自动触发
    pre_delete                  # django的modal对象删除delete()前，自动触发
    post_delete                 # django的modal对象删除delete()后，自动触发
    m2m_changed  # django的modal中使用m2m字段操作第三张表（add,remove,clear）前后，自动触发
    class_prepared # 程序启动时，检测已注册的app中modal类，对于每一个类，自动触发
Management signals
    pre_migrate                 # 执行migrate命令前，自动触发
    post_migrate                # 执行migrate命令后，自动触发
Request/response signals
    request_started             # 请求到来前，自动触发
    request_finished            # 请求结束后，自动触发
    got_request_exception       # 请求异常后，自动触发
Test signals
    setting_changed             # 使用test测试修改配置文件时，自动触发
    template_rendered           # 使用test测试渲染模板时，自动触发
Database Wrappers
    connection_created          # 创建数据库连接时，自动触发
"""


@receiver(post_save, sender=MessageCenter)
def create_message_center(sender, instance, created, **kwargs):
    # created  创建
    logger.info(f"MessageCenter 信号触发 - 实例ID: {instance.id}, 是否创建: {created}")
    if created:
        try:
            new_message_center = MessageCenter.objects.get(id=instance.id)
            send_user_info = get_auth_user_info(new_message_center.send_user_info.get("nid", None))
            logger.debug("MessageCenter 目标用户 send_user_info=%s", send_user_info)
            if not send_user_info:
                logger.warning("MessageCenter 发送失败：未找到目标用户 message_id=%s", instance.id)
                return "发送失败未找到用户数为0"
            message_center_target_user_list = []
            for uid in send_user_info:
                message_center_target_user_list.append(
                    MessageCenterTargetUser(
                        users_id=uid,
                        messagecenter=new_message_center,
                        is_send=False,
                        creator=new_message_center.creator,
                        modifier=new_message_center.modifier,
                    )
                )
            if message_center_target_user_list:
                MessageCenterTargetUser.objects.bulk_create(message_center_target_user_list)
                logger.info("成功创建 %d 个消息中心目标用户记录 message_id=%s", len(message_center_target_user_list), instance.id)
        except Exception as e:
            logger.exception("创建消息中心目标用户时发生异常 message_id=%s: %s", instance.id, e)


headers = {
    "Content-Type": "application/json; charset=utf-8",
}


@receiver([pre_save], sender=Users)
def prevent_anonymous_user(sender, instance, **kwargs):
    """阻止创建用户名为 AnonymousUser 的用户 - 静默处理，不抛出异常"""
    if instance.username == "AnonymousUser":
        logger.warning("检测到尝试创建 AnonymousUser 用户，已静默阻止保存")
        # 将 username 设置为一个临时标记值，在 post_save 中会删除该记录
        # 这样不会抛出异常，而是静默阻止
        instance.username = "_ANONYMOUS_USER_BLOCKED_"
        return


@receiver([post_save], sender=Users)
def create_update_delete_users(sender, instance, created, **kwargs):
    logger.info(f"Users 信号触发 - 用户ID: {instance.id}, 用户名: {instance.username}, 是否创建: {created}")

    # 如果用户名为标记值（在 pre_save 中被修改），或者用户名为 "AnonymousUser"，删除该用户
    if instance.username == "_ANONYMOUS_USER_BLOCKED_" or instance.username == "AnonymousUser":
        logger.warning(f"检测到不应该创建的用户（ID: {instance.id}, 用户名: {instance.username}），正在删除")
        try:
            # 使用 _raw_delete 或直接删除，但需要先保存 ID
            user_id = instance.id
            # 使用 QuerySet 的 delete()，这会触发信号，但在 delete_users 中会跳过处理
            Users.objects.filter(id=user_id).delete()
            logger.info(f"已删除不应该创建的用户（ID: {user_id}）")
        except Exception as e:
            logger.error(f"删除不应该创建的用户失败: {e}")
        return

    try:
        apps_result = Apps.objects.filter(create_or_update_callback_url="").all()
        payload = json.dumps(
            {"node_type": 4, "action_type": 0 if created else 1, "info": dict(UserSerializer(instance).data)},
            ensure_ascii=False,
        )
        logger.info(f"准备向 {len(apps_result)} 个应用发送用户变更回调")

        for item in apps_result:
            try:
                logger.debug(f"向应用 {item.title} 发送用户变更回调")
                httpx.post(item.create_or_update_callback_url, headers=headers, data=payload)
                logger.info(f"成功向应用 {item.title} 发送用户变更回调")
            except Exception as e:
                logger.warning("向应用 %s 发送用户变更回调失败: %s", item.title, e)
    except Exception as e:
        logger.exception("处理用户变更信号时发生异常: %s", e)


@receiver([post_delete], sender=Users)
def delete_users(sender, instance, **kwargs):
    logger.info(f"Users 删除信号触发 - 用户ID: {instance.id}, 用户名: {instance.username}")

    # 如果用户名为 "AnonymousUser" 或标记值，跳过回调处理
    if instance.username == "AnonymousUser" or instance.username == "_ANONYMOUS_USER_BLOCKED_":
        logger.debug("跳过不应该创建用户的删除回调处理")
        return

    try:
        apps_result = Apps.objects.filter(delete_callback_url="").all()
        payload = json.dumps(
            {"node_type": 4, "action_type": 2, "info": dict(UserSerializer(instance).data)}, ensure_ascii=False
        )
        logger.info(f"准备向 {len(apps_result)} 个应用发送用户删除回调")

        for item in apps_result:
            try:
                logger.debug(f"向应用 {item.title} 发送用户删除回调")
                httpx.post(item.delete_callback_url, headers=headers, data=payload)
                logger.info(f"成功向应用 {item.title} 发送用户删除回调")
            except Exception as e:
                logger.warning("向应用 %s 发送用户删除回调失败: %s", item.title, e)
    except Exception as e:
        logger.exception("处理用户删除信号时发生异常: %s", e)


@receiver([post_save], sender=Tags)
def create_update_tags(sender, instance, created, **kwargs):
    logger.info(f"Tags 信号触发 - 标签ID: {instance.id}, 标签名: {instance.name}, 是否创建: {created}")
    try:
        apps_result = Apps.objects.filter(~Q(create_or_update_callback_url="")).all()
        payload = json.dumps(
            {"node_type": 3, "action_type": 0 if created else 1, "info": dict(TagsSerializer(instance).data)},
            ensure_ascii=False,
        )
        logger.info(f"准备向 {len(apps_result)} 个应用发送标签变更回调")

        for item in apps_result:
            try:
                logger.debug(f"向应用 {item.title} 发送标签变更回调")
                httpx.post(item.create_or_update_callback_url, headers=headers, data=payload)
                logger.info(f"成功向应用 {item.title} 发送标签变更回调")
            except Exception as e:
                logger.warning("向应用 %s 发送标签变更回调失败: %s", item.title, e)
    except Exception as e:
        logger.exception("处理标签变更信号时发生异常: %s", e)


@receiver([post_delete], sender=Tags)
def delete_tags(sender, instance, **kwargs):
    logger.info(f"Tags 删除信号触发 - 标签ID: {instance.id}, 标签名: {instance.name}")
    try:
        apps_result = Apps.objects.filter(delete_callback_url="").all()
        payload = json.dumps(
            {"node_type": 3, "action_type": 2, "info": dict(TagsSerializer(instance).data)}, ensure_ascii=False
        )
        logger.info(f"准备向 {len(apps_result)} 个应用发送标签删除回调")

        for item in apps_result:
            try:
                logger.debug(f"向应用 {item.title} 发送标签删除回调")
                httpx.post(item.delete_callback_url, headers=headers, data=payload)
                logger.info(f"成功向应用 {item.title} 发送标签删除回调")
            except Exception as e:
                logger.warning("向应用 %s 发送标签删除回调失败: %s", item.title, e)
    except Exception as e:
        logger.exception("处理标签删除信号时发生异常: %s", e)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import logging

from django.conf import settings
from django.core.cache import cache
from django.db import connection
from django.db.utils import OperationalError, ProgrammingError

from django_base_ai.utils.validator import CustomValidationError

# 配置日志
logger = logging.getLogger(__name__)

dispatch_db_type = getattr(settings, "DISPATCH_DB_TYPE", "memory")  # redis


def is_tenants_mode():
    """
    判断是否为租户模式
    :return:
    """
    is_tenant = hasattr(connection, "tenant") and connection.tenant.schema_name
    logger.debug(f"租户模式检查: {is_tenant}")
    return is_tenant


# ================================================= #
# ******************** 初始化 ******************** #
# ================================================= #
# 递归设置字典节点


def get_all_list(queryset=None, dictionary_value=None):
    from django_base_ai.system.models import Dictionary

    logger.debug(f"开始获取字典列表, dictionary_value: {dictionary_value}")
    if queryset is None:
        queryset = Dictionary.objects.filter(status=True, is_value=False)
    # 获取指定节点
    if dictionary_value is not None:
        queryset = queryset.filter(status=True, value=dictionary_value)
    data = []
    for instance in queryset:
        item = {
            "id": instance.id,
            "label": instance.label,
            "value": instance.value,
            "type": instance.type,
            "color": instance.color,
            "icon": instance.icon,
            "children": [],
        }
        _childs = Dictionary.objects.filter(parent=instance, status=True)
        if _childs:
            item["children"] = get_all_list(_childs, None)
        data.append(item)
    logger.debug(f"获取字典列表完成, 共 {len(data)} 条记录")
    return data


def _get_all_dictionary():
    try:
        from django_base_ai.system.models import Dictionary

        logger.debug("开始从数据库获取所有字典配置")
        queryset = Dictionary.objects.filter(status=True, is_value=False)
        data = []
        for instance in queryset:
            data.append(
                {
                    "id": instance.id,
                    "label": instance.label,
                    "value": instance.value,
                    "type": instance.type,
                    "color": instance.color,
                    "icon": instance.icon,
                    "remark": instance.remark,
                    "children": list(
                        Dictionary.objects.filter(parent=instance.id, status=True)
                        .filter(status=1)
                        .values("id", "label", "value", "type", "color", "icon", "remark")
                    ),
                }
            )
        result = {ele.get("value"): ele for ele in data}
        logger.info(f"从数据库获取字典配置完成, 共 {len(result)} 个字典项")
        return result
    except (ProgrammingError, OperationalError) as e:
        # 如果是表不存在的异常（例如在执行makemigrations时），返回空字典
        error_str = str(e)
        if "doesn't exist" in error_str or "Table" in error_str:
            logger.debug("数据库表尚未创建，返回空字典（可能正在执行迁移）")
            return {}
        raise
    except Exception as e:
        # 如果是 AppRegistryNotReady 异常，返回空字典
        error_str = str(e)
        if "AppRegistryNotReady" in str(type(e).__name__) or "Models aren't loaded yet" in error_str:
            logger.debug("Django 应用尚未初始化，返回空字典")
            return {}
        raise


def _get_all_system_config():
    try:
        data = {}
        from django_base_ai.system.models import SystemConfig

        logger.debug("开始从数据库获取所有系统配置")
        system_config_obj = (
            SystemConfig.objects.filter(parent_id__isnull=False)
            .values("parent__key", "key", "value", "form_item_type")
            .order_by("sort")
        )
        for system_config in system_config_obj:
            value = system_config.get("value", "")
            if value and system_config.get("form_item_type") == 7:
                value = value[0].get("url")
            data[f"{system_config.get('parent__key')}.{system_config.get('key')}"] = value
        logger.info(f"从数据库获取系统配置完成, 共 {len(data)} 个配置项")
        return data
    except (ProgrammingError, OperationalError) as e:
        # 如果是表不存在的异常（例如在执行makemigrations时），返回空字典
        error_str = str(e)
        if "doesn't exist" in error_str or "Table" in error_str:
            logger.debug("数据库表尚未创建，返回空字典（可能正在执行迁移）")
            return {}
        raise
    except Exception as e:
        # 如果是 AppRegistryNotReady 异常，返回空字典
        error_str = str(e)
        if "AppRegistryNotReady" in str(type(e).__name__) or "Models aren't loaded yet" in error_str:
            logger.debug("Django 应用尚未初始化，返回空字典")
            return {}
        logger.error(f"获取系统配置失败: {str(e)}", exc_info=True)
        return {}


def init_dictionary():
    """
    初始化字典配置
    :return:
    """
    logger.info(f"开始初始化字典配置, 存储类型: {dispatch_db_type}")
    try:
        if dispatch_db_type == "redis":
            logger.debug("使用Redis存储字典配置")
            dictionary_data = _get_all_dictionary()
            cache.set("init_dictionary", dictionary_data)
            logger.info("字典配置已存储到Redis缓存")
            return
        else:
            logger.debug("使用内存存储字典配置")
            settings.DICTIONARY_CONFIG = _get_all_dictionary()
            logger.info("字典配置已存储到内存")
    except Exception as e:
        # _get_all_dictionary 已经处理了表不存在的情况，这里只是做最后的容错
        error_str = str(e)
        if isinstance(e, (ProgrammingError, OperationalError)) and (
            "doesn't exist" in error_str or "Table" in error_str
        ):
            logger.debug("数据库表尚未创建，跳过字典配置初始化（可能正在执行迁移）")
            if dispatch_db_type == "redis":
                cache.set("init_dictionary", {})
            else:
                settings.DICTIONARY_CONFIG = {}
        else:
            logger.error(f"初始化字典配置失败: {str(e)}", exc_info=True)
    return


def init_system_config():
    """
    初始化系统配置
    :param name:
    :return:
    """
    logger.info(f"开始初始化系统配置, 存储类型: {dispatch_db_type}")
    try:
        if dispatch_db_type == "redis":
            logger.debug("使用Redis存储系统配置")
            system_config_data = _get_all_system_config()
            cache.set("init_system_config", system_config_data)
            logger.info("系统配置已存储到Redis缓存")
            return
        else:
            logger.debug("使用内存存储系统配置")
            settings.SYSTEM_CONFIG = _get_all_system_config()
            logger.info("系统配置已存储到内存")
    except Exception as e:
        # _get_all_system_config 已经处理了表不存在的情况，这里只是做最后的容错
        error_str = str(e)
        if isinstance(e, (ProgrammingError, OperationalError)) and (
            "doesn't exist" in error_str or "Table" in error_str
        ):
            logger.debug("数据库表尚未创建，跳过系统配置初始化（可能正在执行迁移）")
            if dispatch_db_type == "redis":
                cache.set("init_system_config", {})
            else:
                settings.SYSTEM_CONFIG = {}
        else:
            logger.error(f"初始化系统配置失败: {str(e)}", exc_info=True)
    return


def refresh_dictionary():
    """
    刷新字典配置
    :return:
    """
    try:
        logger.info("刷新字典配置")
        if dispatch_db_type == "redis":
            dictionary_data = _get_all_dictionary()
            cache.set("init_dictionary", dictionary_data)
            logger.info("字典配置已刷新到Redis缓存")
            return
        else:
            settings.DICTIONARY_CONFIG = _get_all_dictionary()
            logger.info("字典配置已刷新到内存")
    except Exception as e:
        # 如果是 AppRegistryNotReady 异常，静默失败
        error_str = str(e)
        if "AppRegistryNotReady" in str(type(e).__name__) or "Models aren't loaded yet" in error_str:
            logger.debug("Django 应用尚未初始化，跳过字典配置刷新")
            return
        raise


def refresh_system_config():
    """
    刷新系统配置
    :return:
    """
    logger.info("刷新系统配置")
    if dispatch_db_type == "redis":
        system_config_data = _get_all_system_config()
        cache.set("init_system_config", system_config_data)
        logger.info("系统配置已刷新到Redis缓存")
    else:
        settings.SYSTEM_CONFIG = _get_all_system_config()
        logger.info("系统配置已刷新到内存")


# ================================================= #
# ******************** 字典管理 ******************** #
# ================================================= #
def get_dictionary_config(schema_name=None):
    """
    获取字典所有配置
    :param schema_name: 对应字典配置的租户schema_name值
    :return:
    """
    logger.debug(f"开始获取字典配置, schema_name: {schema_name}")
    dictionary_config = {}
    if dispatch_db_type == "redis":
        init_dictionary_data = cache.get("init_dictionary")
        if not init_dictionary_data:
            logger.warning("Redis中没有找到字典配置，开始刷新")
            refresh_dictionary()
        result = cache.get("init_dictionary") or {}
        logger.debug(f"从Redis获取字典配置完成, 共 {len(result)} 个配置")
        return result
    if not settings.DICTIONARY_CONFIG:
        logger.warning("内存中没有字典配置，开始刷新")
        refresh_dictionary()
    else:
        dictionary_config = settings.DICTIONARY_CONFIG
    result = dictionary_config or {}
    logger.debug(f"从内存获取字典配置完成, 共 {len(result)} 个配置")
    return result


def get_dictionary_values(key, schema_name=None):
    """
    获取字典数据数组
    :param key: 对应字典配置的key值(字典编号)
    :param schema_name: 对应字典配置的租户schema_name值
    :return:
    """
    try:
        logger.debug(f"获取字典值, key: {key}, schema_name: {schema_name}")
        if dispatch_db_type == "redis":
            dictionary_config = cache.get("init_dictionary")
            if not dictionary_config:
                logger.debug("缓存中没有字典配置，开始刷新")
                refresh_dictionary()
                dictionary_config = cache.get("init_dictionary")
            result = dictionary_config.get(key)
            logger.debug(f"从Redis获取字典值完成, key: {key}, 结果存在: {result is not None}")
            return result
        dictionary_config = get_dictionary_config(schema_name)
        result = dictionary_config.get(key)
        logger.debug(f"从内存获取字典值完成, key: {key}, 结果存在: {result is not None}")
        return result
    except Exception as e:
        # 如果是 AppRegistryNotReady 异常，返回 None
        error_str = str(e)
        if "AppRegistryNotReady" in str(type(e).__name__) or "Models aren't loaded yet" in error_str:
            logger.debug(f"Django 应用尚未初始化，返回 None, key: {key}")
            return None
        raise


def get_dictionary_label(key, name, schema_name=None):
    """
    获取获取字典label值
    :param key: 字典管理中的key值(字典编号)
    :param name: 对应字典配置的value值
    :param schema_name: 对应字典配置的租户schema_name值
    :return:
    """
    logger.debug(f"获取字典标签, key: {key}, name: {name}, schema_name: {schema_name}")
    res = get_dictionary_values(key, schema_name) or {}
    children = res.get("children", [])
    for ele in children:
        if ele.get("value") == str(name):
            label = ele.get("label")
            logger.debug(f"找到匹配的字典标签: {label}")
            return label
    logger.debug(f"未找到匹配的字典标签, key: {key}, name: {name}")
    return ""


# ================================================= #
# ******************** 系统配置 ******************** #
# ================================================= #
def get_system_config(schema_name=None):
    """
    获取系统配置中所有配置
    1.只传父级的key，返回全部子级，{ "父级key.子级key" : "值" }
    2."父级key.子级key"，返回子级值
    :param schema_name: 对应字典配置的租户schema_name值
    :return:
    """
    logger.debug(f"开始获取系统配置, schema_name: {schema_name}")
    system_config = {}
    if dispatch_db_type == "redis":
        init_system_config_data = cache.get("init_system_config")
        if not init_system_config_data:
            logger.warning("Redis中没有找到系统配置，开始刷新")
            refresh_system_config()
        result = cache.get("init_system_config") or {}
        logger.debug(f"从Redis获取系统配置完成, 共 {len(result)} 个配置")
        return result
    if not settings.SYSTEM_CONFIG:
        logger.warning("内存中没有系统配置，开始刷新")
        refresh_system_config()
    else:
        system_config = settings.SYSTEM_CONFIG
    logger.debug(f"从内存获取系统配置完成, 共 {len(system_config)} 个配置")
    return system_config


def get_system_config_values(key, schema_name=None):
    """
    获取系统配置数据数组
    :param key: 对应系统配置的key值(字典编号)
    :param schema_name: 对应系统配置的租户schema_name值
    :return:
    """
    logger.debug(f"获取系统配置值, key: {key}, schema_name: {schema_name}")
    if dispatch_db_type == "redis":
        system_config = cache.get("init_system_config")
        if not system_config:
            logger.debug("缓存中没有系统配置，开始刷新")
            refresh_system_config()
            system_config = cache.get("init_system_config")
        result = system_config.get(key)
        logger.debug(f"从Redis获取系统配置值完成, key: {key}, 结果存在: {result is not None}")
        return result
    system_config = get_system_config(schema_name)
    result = system_config.get(key)
    logger.debug(f"从内存获取系统配置值完成, key: {key}, 结果存在: {result is not None}")
    return result


def get_system_config_values_to_dict(key, schema_name=None):
    """
    获取系统配置数据并转换为字典 **仅限于数组类型系统配置
    :param key: 对应系统配置的key值(字典编号)
    :param schema_name: 对应系统配置的租户schema_name值
    :return:
    """
    logger.debug(f"将系统配置转换为字典, key: {key}, schema_name: {schema_name}")
    values_dict = {}
    config_values = get_system_config_values(key, schema_name)
    if not isinstance(config_values, list):
        logger.error(f"系统配置值不是列表类型, key: {key}")
        raise CustomValidationError("该方式仅限于数组类型系统配置")
    for ele in get_system_config_values(key, schema_name):
        values_dict[ele.get("key")] = ele.get("value")
    logger.debug(f"系统配置转换为字典完成, key: {key}, 共 {len(values_dict)} 个条目")
    return values_dict


def get_system_config_label(key, name, schema_name=None):
    """
    获取获取系统配置label值
    :param key: 系统配置中的key值(字典编号)
    :param name: 对应系统配置的value值
    :param schema_name: 对应系统配置的租户schema_name值
    :return:
    """
    logger.debug(f"获取系统配置标签, key: {key}, name: {name}, schema_name: {schema_name}")
    children = get_system_config_values(key, schema_name) or []
    for ele in children:
        if ele.get("value") == str(name):
            label = ele.get("label")
            logger.debug(f"找到匹配的系统配置标签: {label}")
            return label
    logger.debug(f"未找到匹配的系统配置标签, key: {key}, name: {name}")
    return ""


# 获取字典数据-转化成tuple 用于字典choices
def get_dictionary_choices(dictionarg_key="") -> tuple:
    try:
        logger.debug(f"将字典数据转换为choices, key: {dictionarg_key}")
        dictionary_values = get_dictionary_values(dictionarg_key) or {}
        dictionary_children = dictionary_values.get("children", [])

        result = []
        for item in dictionary_children:
            value = item.get("value", 0)
            label = item.get("label", "未知")
            try:
                # 尝试转换为int
                int_value = int(value)
                result.append((int_value, label))
            except (ValueError, TypeError):
                # 如果无法转换为int，跳过该项
                logger.warning(f"字典值无法转换为int, key: {dictionarg_key}, value: {value}, 已跳过")
                continue

        result = tuple(result)
        logger.debug(f"字典choices转换完成, key: {dictionarg_key}, 共 {len(result)} 个选项")
        return result
    except Exception as e:
        # 如果是 AppRegistryNotReady 异常，静默返回空元组
        error_str = str(e)
        if "AppRegistryNotReady" in str(type(e).__name__) or "Models aren't loaded yet" in error_str:
            logger.debug(f"Django 应用尚未初始化，返回空元组, key: {dictionarg_key}")
            return ()
        logger.error(f"获取字典choices失败, key: {dictionarg_key}, 错误: {str(e)}", exc_info=True)
        return ()


# 获取导出excel下拉选择项
def get_import_dictionary_choices(dictionarg_key="") -> dict:
    try:
        logger.debug(f"获取导出excel下拉选择项, key: {dictionarg_key}")
        dictionary_values = get_dictionary_values(dictionarg_key) or {}
        dictionary_children = dictionary_values.get("children", [])
        choices_infos = {}
        for item in dictionary_children:
            value = item.get("value", 0)
            try:
                int_value = int(value)
                choices_infos.update({item.get("label", "未知"): int_value})
            except (ValueError, TypeError):
                logger.warning(f"字典值无法转换为int, key: {dictionarg_key}, value: {value}, 已跳过")
                continue
        logger.debug(f"导出下拉选择项获取完成, key: {dictionarg_key}, 共 {len(choices_infos)} 个选项")
        return choices_infos
    except Exception as e:
        logger.error(f"获取导出下拉选择项失败, key: {dictionarg_key}, 错误: {str(e)}", exc_info=True)
        return {}


# 通过key 获取字典 返回dict {value:label}
def get_dictionary_dict_result(dictionarg_key="") -> dict:
    try:
        logger.debug(f"获取字典字典结果, key: {dictionarg_key}")
        dictionary_values = get_dictionary_values(dictionarg_key) or {}
        dictionary_children = dictionary_values.get("children", [])
        dict_result = {}
        for item in dictionary_children:
            value = item.get("value", 0)
            label = item.get("label", "未知")
            dict_result.update({value: label})
        logger.debug(f"字典结果获取完成, key: {dictionarg_key}, 共 {len(dict_result)} 个条目")
        return dict_result
    except Exception as e:
        logger.error(f"获取字典结果失败, key: {dictionarg_key}, 错误: {str(e)}", exc_info=True)
        return {}


def get_all_tree_choice_recursive(dictionary_list: list, save_list: list) -> None:
    logger.debug(f"递归处理字典树, 当前级别: {len(dictionary_list)} 个节点")
    for i in dictionary_list:
        save_list.append({"label": i["label"], "value": i["value"]})
        if i["children"]:
            get_all_tree_choice_recursive(i["children"], save_list)
    return None


def get_dictionary_tree_all_choices(dictionarg_key="") -> dict:
    try:
        logger.debug(f"获取字典树所有选项, key: {dictionarg_key}")
        dictionarg_list = get_all_list(dictionary_value=dictionarg_key)
        save_list = []
        get_all_tree_choice_recursive(dictionarg_list, save_list)
        result = {i["value"]: i["label"] for i in save_list}
        logger.debug(f"字典树所有选项获取完成, key: {dictionarg_key}, 共 {len(result)} 个选项")
        return result
    except Exception as e:
        logger.error(f"获取字典树所有选项失败, key: {dictionarg_key}, 错误: {str(e)}", exc_info=True)
        print(e.args)
        return []



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : role.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 角色 CRUD、权限与初始化。
import logging

from rest_framework import serializers
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated

from django_base_ai.system.models import Dept, Menu, MenuButton, Role
from django_base_ai.system.views.dept import DeptSerializer
from django_base_ai.system.views.menu import MenuSerializer
from django_base_ai.system.views.menu_button import MenuButtonSerializer
from django_base_ai.utils.json_response import DetailResponse
from django_base_ai.utils.permission import OpenApiPermission
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.validator import CustomUniqueValidator
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class RoleSerializer(CustomModelSerializer):
    """
    角色-序列化器
    """

    class Meta:
        model = Role
        fields = "__all__"
        read_only_fields = ["id"]


class RoleInitSerializer(CustomModelSerializer):
    """
    初始化获取数信息(用于生成初始化json文件)
    """

    class Meta:
        model = Role
        fields = ["name", "key", "sort", "status", "admin", "data_range", "remark", "creator", "dept_belong_id"]
        read_only_fields = ["id"]
        extra_kwargs = {"creator": {"write_only": True}, "dept_belong_id": {"write_only": True}}


class RoleCreateUpdateSerializer(CustomModelSerializer):
    """
    角色管理 创建/更新时的列化器
    """

    menu = MenuSerializer(many=True, read_only=True)
    dept = DeptSerializer(many=True, read_only=True)
    permission = MenuButtonSerializer(many=True, read_only=True)
    key = serializers.CharField(
        max_length=50, validators=[CustomUniqueValidator(queryset=Role.objects.all(), message="权限字符必须唯一")]
    )
    name = serializers.CharField(max_length=50, validators=[CustomUniqueValidator(queryset=Role.objects.all())])

    def validate(self, attrs: dict):
        return super().validate(attrs)

    def save(self, **kwargs):
        is_superuser = self.request.user.is_superuser
        if not is_superuser:
            self.validated_data.pop("admin")
        data = super().save(**kwargs)
        data.dept.set(self.initial_data.get("dept", []))
        data.menu.set(self.initial_data.get("menu", []))
        data.permission.set(self.initial_data.get("permission", []))
        return data

    class Meta:
        model = Role
        fields = "__all__"


class MenuPermissonSerializer(CustomModelSerializer):
    """
    菜单的按钮权限
    """

    menu_permission = serializers.SerializerMethodField()

    def get_menu_permission(self, instance):
        is_superuser = self.request.user.is_superuser
        if is_superuser:
            queryset = MenuButton.objects.filter(menu__id=instance.id)
        else:
            menu_permission_id_list = self.request.user.role.values_list("permission", flat=True)
            queryset = MenuButton.objects.filter(id__in=menu_permission_id_list, menu__id=instance.id)
        serializer = MenuButtonSerializer(queryset, many=True, read_only=True)
        return serializer.data

    class Meta:
        model = Menu
        fields = ["id", "parent", "name", "meta", "menu_permission", "path"]


class RoleViewSet(CustomModelViewSet):
    """
    角色管理接口
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = Role.objects.all()
    serializer_class = RoleSerializer
    create_serializer_class = RoleCreateUpdateSerializer
    update_serializer_class = RoleCreateUpdateSerializer
    search_fields = ["name", "key"]
    # extra_filter_backends = []

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def role_get_menu(self, request):
        """根据当前用户的角色返回角色拥有的菜单"""
        logger.info(f"用户 {request.user.username} 请求获取角色菜单")
        try:
            is_superuser = request.user.is_superuser
            is_admin = request.user.role.values_list("admin", flat=True)
            if is_superuser or True in is_admin:
                queryset = Menu.objects.all()
                logger.info(f"用户 {request.user.username} 为超级用户或管理员，返回所有菜单")
            else:
                menu_id_list = request.user.role.values_list("menu", flat=True)
                queryset = Menu.objects.filter(id__in=menu_id_list)
                logger.info(f"用户 {request.user.username} 返回角色菜单，菜单数量: {len(menu_id_list)}")
            # queryset = self.filter_queryset(queryset)
            serializer = MenuPermissonSerializer(queryset, many=True, request=request)
            logger.info(f"用户 {request.user.username} 菜单获取成功")
            return DetailResponse(data=serializer.data)
        except Exception as e:
            logger.error(f"用户 {request.user.username} 获取角色菜单时发生异常: {e}")
            logger.exception(e)
            return DetailResponse(data=[], msg="获取菜单失败")

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def data_scope(self, request):
        is_superuser = request.user.is_superuser
        role_queryset = Role.objects.filter(users__id=request.user.id).values_list("data_range", flat=True)
        if is_superuser:
            data = [
                {"value": 0, "label": "仅本人数据权限"},
                {"value": 1, "label": "本部门及以下数据权限"},
                {"value": 2, "label": "本部门数据权限"},
                {"value": 3, "label": "全部数据权限"},
                {"value": 4, "label": "自定义数据权限"},
            ]
        else:
            data = []
            data_range_list = list(set(role_queryset))
            for item in data_range_list:
                if item == 0:
                    data = [{"value": 0, "label": "仅本人数据权限"}]
                elif item == 1:
                    data = [
                        {"value": 0, "label": "仅本人数据权限"},
                        {"value": 1, "label": "本部门及以下数据权限"},
                        {"value": 2, "label": "本部门数据权限"},
                    ]
                elif item == 2:
                    data = [{"value": 0, "label": "仅本人数据权限"}, {"value": 2, "label": "本部门数据权限"}]
                elif item == 3:
                    data = [
                        {"value": 0, "label": "仅本人数据权限"},
                        {"value": 3, "label": "全部数据权限"},
                    ]
                elif item == 4:
                    data = [{"value": 0, "label": "仅本人数据权限"}, {"value": 4, "label": "自定义数据权限"}]
                else:
                    data = []
        return DetailResponse(data=data)

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def data_scope_dept(self, request):
        """根据当前角色获取部门信息"""
        is_superuser = request.user.is_superuser
        if is_superuser:
            queryset = Dept.objects.values("id", "name", "parent")
        else:
            dept_list = request.user.role.values_list("dept", flat=True)
            queryset = Dept.objects.filter(id__in=dept_list).values("id", "name", "parent")
        return DetailResponse(data=queryset)

    @action(methods=["GET"], detail=False, permission_classes=[OpenApiPermission], authentication_classes=[])
    def all_role_info(self, request):
        """
        仅用于第三方调用
        :param request:
        :return:
        """
        return self.list(request)



#!/usr/bin/env python
"""
@Project ：django_base_ai
@File    ：cx_aes.py
@Author  ：congxing.wang
@Date    ：2026/02/25
@Desc    ：AES 加解密
"""

import binascii

from Crypto.Cipher import AES


class AESCipher:
    def __init__(self, secret_key: str, iv: str):
        self.BLOCK_SIZE = AES.block_size
        self.pad = lambda s: (
            s
            + (self.BLOCK_SIZE - len(s.encode()) % self.BLOCK_SIZE)
            * chr(self.BLOCK_SIZE - len(s.encode()) % self.BLOCK_SIZE)
        )
        self.unpad = lambda s: s[: -ord(s[len(s) - 1 :])]
        self.key = secret_key
        self.iv = iv

    def encrypt(self, text):
        text = self.pad(text).encode()
        cipher = AES.new(key=self.key.encode(), mode=AES.MODE_CBC, IV=self.iv.encode())
        encrypted_text = cipher.encrypt(text)
        return encrypted_text.hex().upper()

    def decrypt(self, encrypted_text):
        cipher = AES.new(key=self.key.encode(), mode=AES.MODE_CBC, IV=self.iv.encode())
        decrypted_text = cipher.decrypt(binascii.a2b_hex(encrypted_text))
        return self.unpad(decrypted_text).decode("utf-8")


if __name__ == "__main__":
    secret_key = "|dSQG|YRSZ+Q%3*FDo*^G(L]8{D{(4C_"
    # 获取前16位作为IV
    iv = secret_key[:16]
    print("IV:", iv)  # 输出: |dSQG|YRSZ+Q%3*F
    # 获取后16位作为key
    key = secret_key[-16:]
    print("Key:", key)  # 输出: Do*^G(L]8{D{(4C_
    print(AESCipher(secret_key=key, iv=iv).encrypt("test01"))



"""
pytest 配置与 fixture（Rules/python.mdc: 使用 pytest 做测试）。
"""

import os
import sys

import django

# 确保项目根在 path 中，并配置 Django
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "DjangoBaseAi.settings")
django.setup()


import pytest
from rest_framework.test import APIClient

from django_base_ai.system.models import Users


# API 根路径（与 DjangoBaseAi/urls.py 中 path 一致）
API_BASE = "/base/api/system"


@pytest.fixture
def api_client():
    """未认证的 DRF APIClient。"""
    return APIClient()


@pytest.fixture
def test_user(db):
    """测试用用户，用于需要登录的接口。"""
    user = Users(
        username="test_api_user",
        name="测试用户",
        is_staff=False,
        is_active=True,
    )
    user.set_password("TestPassword123")
    user.save()
    return user


@pytest.fixture
def authenticated_client(api_client, test_user):
    """已认证的 APIClient（force_authenticate）。"""
    api_client.force_authenticate(user=test_user)
    return api_client



#!/usr/bin/env python
# -*- coding: UTF-8 -*-


from django.db import DataError
from rest_framework.exceptions import APIException
from rest_framework.validators import UniqueValidator


class CustomValidationError(APIException):
    """
    继承并重写验证器返回的结果,避免暴露字段
    """

    def __init__(self, detail):
        self.detail = detail


def qs_exists(queryset):
    try:
        return queryset.exists()
    except (TypeError, ValueError, DataError):
        return False


def qs_filter(queryset, **kwargs):
    try:
        return queryset.filter(**kwargs)
    except (TypeError, ValueError, DataError):
        return queryset.none()


class CustomUniqueValidator(UniqueValidator):
    """
    继承,重写必填字段的验证器结果,防止字段暴露
    """

    def filter_queryset(self, value, queryset, field_name):
        """
        Filter the queryset to all instances matching the given attribute.
        """
        filter_kwargs = {f"{field_name}__{self.lookup}": value}
        return qs_filter(queryset, **filter_kwargs)

    def exclude_current_instance(self, queryset, instance):
        """
        If an instance is being updated, then do not include
        that instance itself as a uniqueness conflict.
        """
        if instance is not None:
            return queryset.exclude(pk=instance.pk)
        return queryset

    def __call__(self, value, serializer_field):
        # Determine the underlying model field name. This may not be the
        # same as the serializer field name if `source=<>` is set.
        field_name = serializer_field.source_attrs[-1]
        # Determine the existing instance, if this is an update operation.
        instance = getattr(serializer_field.parent, "instance", None)

        queryset = self.queryset
        queryset = self.filter_queryset(value, queryset, field_name)
        queryset = self.exclude_current_instance(queryset, instance)
        if qs_exists(queryset):
            raise CustomValidationError(f"{self.message}（{value}）")

    def __repr__(self):
        return super().__repr__()



"""节假日 HolidayView 接口单元测试。"""

import pytest
from rest_framework import status


pytestmark = [pytest.mark.django_db]

API = "/base/api/system/holiday/"


def test_holiday_get_200(api_client):
    """GET holiday/ 无需认证，返回 200 且 code=2000（依赖 holidays 模块与数据正常）。"""
    response = api_client.get(API)
    assert response.status_code == status.HTTP_200_OK
    data = response.json()
    assert data.get("code") == 2000, data
    assert "data" in data



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : im_chat_to_group.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : IM 群组会话与成员管理。

import logging

from django_base_ai.system.models import IMChatToGroup
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class IMChatToGroupSerializer(CustomModelSerializer):
    class Meta:
        model = IMChatToGroup
        fields = "__all__"
        read_only_fields = ["id"]


class IMChatToGroupCreateUpdateSerializer(CustomModelSerializer):
    class Meta:
        model = IMChatToGroup
        fields = "__all__"


class IMChatToGroupViewSet(CustomModelViewSet):
    """
    群聊
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = IMChatToGroup.objects.all()
    serializer_class = IMChatToGroupSerializer
    # extra_filter_backends = []
    filter_fields = ["im_session"]



#!/usr/bin/env python
"""
@Project ：django_base_ai
@File    ：model_to_markdown.py
@Author  ：congxing.wang
@Date    ：2026/02/25
@Desc    ：Django 模型转 Markdown 文档
"""

import logging
import os
import sys
from collections import OrderedDict

import django

logger = logging.getLogger(__name__)
from django.apps import apps
from django.db import connection

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "DjangoBaseAi.settings")
django.setup()

args = sys.argv

models = apps.all_models


def get_app_models(arg: str, all_models: OrderedDict) -> OrderedDict:
    """
    arg is command line first value passed
    """
    return all_models if arg == "all" else all_models[arg]


file_path = args[2] if len(args) == 3 else "fields_detail.md"

if not file_path.endswith(".md"):
    logger.error("输出路径须为 .md 文件，当前: %s", file_path)
    sys.exit(1)

try:
    arg = args[1]
    all_models = get_app_models(arg, models)
    # all_models is an OrderDict
except IndexError:
    all_models = models.values()

markdown_lines = []

table_name = """\n\n## {0}--{1}\n\n"""
table_header = """| 字段名 | 数据类型 | 允许为空 | PK | 字段说明 |
| ---- | ---- | ---- | --- | ---- |
"""
field_desc_str = "| {0} | {1} | {2} | {3} | {4} |"
new_line_str = "\n"


def get_models_fields() -> None:
    """
    get all models and fields,
    format field description into field_desc_str
    and write in file fields_detail.md by default
    if given a specific app name, and file name,
    write fields into a specific file
    :return:
    :rtype:
    """
    for model_odict in all_models:
        for model in model_odict.values():
            markdown_lines.append(table_name.format(model.__name__, model._meta.db_table))
            fields_list = model._meta.get_fields()
            markdown_lines.append(table_header)
            for field in fields_list:
                field_name = field.name
                field_type = field.db_type(connection)
                field_null = field.null
                try:
                    field_limit = field.primary_key
                except AttributeError:
                    field_limit = False
                try:
                    field_verbose = field.verbose_name
                except AttributeError:
                    field_verbose = " "
                _field_str = field_desc_str.format(
                    str(field_name),
                    str(field_type),
                    str(field_null),
                    str(field_limit),
                    str(field_verbose),
                )
                print("_field_str: ", _field_str, end=new_line_str)
                markdown_lines.append(_field_str + new_line_str)
            markdown_lines.append(new_line_str * 2)


def write_to_md(file_path: str) -> None:
    """
    write fields detail into a markdown file,
    file_path could be specific
    :param file_path:
    :type file_path:
    :return:
    :rtype:
    """
    with open(file_path, "w") as fp:
        print("prepare to write markdown file")
        print(len(markdown_lines))
        fp.writelines(markdown_lines)


usage = """
usage:
$ python model_fields.py
---> fields_detail.md
$ python model_fields.py api_home
---> fields_detail.md  # include all models in api_home module
$ python models_fields.py all ~/docs/model_field.md
---> ~/docs/model_field.md  # include all models and output a specific markdown file
$ python model_fields.py company company_models.md
---> company_models.md  # include all models in company models
     and output a specific markdown file
"""


def main(file_path: str) -> None:
    """
    run this function by default,
    will call the get and write method
    :param file_path:
    :type file_path:
    :return:
    :rtype:
    """
    get_models_fields()
    write_to_md(file_path)


if __name__ == "__main__":
    main(file_path)



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : dept.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 部门 CRUD、懒加载树、导出等。
import logging

import numpy as np
import pandas as pd
from django.db.models import Q
from rest_framework import serializers
from rest_framework.decorators import action
from rest_framework.permissions import IsAdminUser, IsAuthenticated

from django_base_ai.system.models import Dept, Users
from django_base_ai.utils.filters import LazyLoadFilter
from django_base_ai.utils.json_response import DetailResponse, ErrorResponse, SuccessResponse
from django_base_ai.utils.permission import AnonymousUserPermission, OpenApiPermission
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class DeptSerializer(CustomModelSerializer):
    """
    部门-序列化器
    """

    parent_name = serializers.CharField(read_only=True, source="parent.name")
    status_label = serializers.SerializerMethodField()
    has_children = serializers.SerializerMethodField()
    owner_user = serializers.SerializerMethodField()

    def get_status_label(self, obj: Dept):
        status = "启用" if obj.status else "禁用"
        logger.debug(f"部门 {obj.name} 状态: {status}")
        return status

    def get_has_children(self, obj: Dept):
        count = Dept.objects.filter(parent_id=obj.id).count()
        logger.debug(f"部门 {obj.name} 子部门数量: {count}")
        return count

    def get_owner_user(self, obj: Dept):
        if not obj.owner_user:
            logger.debug(f"部门 {obj.name} 无负责人")
            return None
        logger.debug(f"部门 {obj.name} 负责人: {obj.owner_user.name}")
        return {"id": obj.owner_user.id, "name": obj.owner_user.name}

    class Meta:
        model = Dept
        fields = "__all__"
        read_only_fields = ["id"]


class DeptImportSerializer(CustomModelSerializer):
    """
    部门-导入-序列化器
    """

    class Meta:
        model = Dept
        fields = "__all__"
        read_only_fields = ["id"]


class DeptInitSerializer(CustomModelSerializer):
    """
    递归深度获取数信息(用于生成初始化json文件)
    """

    children = serializers.SerializerMethodField()

    def get_children(self, obj: Dept):
        logger.debug(f"获取部门 {obj.name} 的子部门")
        data = []
        instance = Dept.objects.filter(parent_id=obj.id)
        if instance:
            serializer = DeptInitSerializer(instance=instance, many=True)
            data = serializer.data
        logger.debug(f"部门 {obj.name} 子部门数量: {len(data)}")
        return data

    def save(self, **kwargs):
        logger.info(f"开始保存部门初始化数据: {self.initial_data.get('name', '未知')}")
        instance = super().save(**kwargs)
        logger.info(f"部门 {instance.name} 基础信息已保存，ID: {instance.id}")
        children = self.initial_data.get("children")
        if children:
            for menu_data in children:
                menu_data["parent"] = instance.id
                filter_data = {"name": menu_data["name"], "parent": menu_data["parent"]}
                if "key" in menu_data:
                    filter_data["key"] = menu_data["key"]
                instance_obj = Dept.objects.filter(**filter_data).first()
                if instance_obj and not self.initial_data.get("reset"):
                    continue
                serializer = DeptInitSerializer(instance_obj, data=menu_data, request=self.request)
                serializer.is_valid(raise_exception=True)
                serializer.save()
                logger.info(f"子部门 {menu_data.get('name')} 已保存")
        logger.info(f"部门 {instance.name} 初始化完成")
        return instance

    class Meta:
        model = Dept
        fields = ["name", "sort", "phone", "email", "status", "parent", "creator", "dept_belong_id", "children", "key"]
        extra_kwargs = {"creator": {"write_only": True}, "dept_belong_id": {"write_only": True}}
        read_only_fields = ["id", "children"]


class DeptCreateUpdateSerializer(CustomModelSerializer):
    """
    部门管理 创建/更新时的列化器
    """

    def create(self, validated_data):
        logger.info(f"开始创建部门: {validated_data.get('name', '未知')}")
        try:
            value = validated_data.get("parent", None)
            if value is None:
                validated_data["parent"] = self.request.user.dept
                logger.info(f"设置部门父级为当前用户部门: {self.request.user.dept}")
            instance = super().create(validated_data)
            instance.dept_belong_id = instance.id
            instance.save()
            logger.info(f"部门创建成功: {instance.name} (ID: {instance.id})")
            return instance
        except Exception as e:
            logger.error(f"创建部门时发生异常: {e}")
            logger.exception(e)
            raise

    class Meta:
        model = Dept
        fields = "__all__"


class DeptLazyFilter(LazyLoadFilter):
    class Meta:
        model = Dept
        fields = ["name", "dept", "parent", "status"]


class DeptViewSet(CustomModelViewSet):
    """
    部门管理接口
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = Dept.objects.all()
    serializer_class = DeptSerializer
    create_serializer_class = DeptCreateUpdateSerializer
    update_serializer_class = DeptCreateUpdateSerializer
    # filter_fields = ["name", "id", "parent"]
    filter_class = DeptLazyFilter
    search_fields = ["name"]
    # extra_filter_backends = []
    import_serializer_class = DeptImportSerializer
    import_field_dict = {
        "name": "部门名称",
        "key": "部门标识",
    }

    def list(self, request, *args, **kwargs):
        logger.info(f"用户 {request.user.username} 请求获取部门列表")
        # 如果懒加载，则只返回父级
        params = request.query_params
        parent = params.get("parent", None)
        if not params or not parent:
            queryset = self.queryset.filter(parent__isnull=True)
            logger.debug("查询顶级部门")
        else:
            queryset = self.queryset.filter(parent=parent)
            logger.debug(f"查询父级部门ID: {parent} 的子部门")
        if request.user.is_staff:
            queryset = queryset.order_by("sort", "create_datetime")
        else:
            queryset = queryset.filter(status=1).order_by("sort", "create_datetime")
        serializer = DeptSerializer(queryset, many=True, request=request)
        data = serializer.data
        logger.info(f"返回部门数量: {len(data)}")
        return SuccessResponse(data=data)

    def retrieve(self, request, *args, **kwargs):
        logger.info(f"用户 {request.user.username} 请求获取部门详情")
        # 重写 否则会很慢
        pk = kwargs.get("pk", 0)
        instance = Dept.objects.filter(id=pk).first()
        if not instance:
            logger.warning(f"未找到部门ID: {pk}")
        else:
            logger.info(f"返回部门: {instance.name}")
        serializer = self.get_serializer(instance)
        return DetailResponse(data=serializer.data)

    def update(self, request, *args, **kwargs):
        logger.info(f"用户 {request.user.username} 请求更新部门")
        # 重写 否则会很慢
        pk = kwargs.get("pk", 0)
        instance = Dept.objects.filter(id=pk).first()
        if not instance:
            logger.warning(f"未找到部门ID: {pk}")
            return ErrorResponse(msg="部门不存在")
        logger.info(f"更新部门: {instance.name}")
        serializer = self.get_serializer(instance, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        logger.info(f"部门 {instance.name} 更新成功")
        return SuccessResponse(msg="修改成功")

    def destroy(self, request, *args, **kwargs):
        logger.info(f"用户 {request.user.username} 请求删除部门")
        pk = kwargs.get("pk", 0)
        instance = Dept.objects.filter(id=pk).first()
        if not instance:
            logger.warning(f"未找到部门ID: {pk}")
            return ErrorResponse(msg="部门不存在")
        dept_name = instance.name
        instance.delete()
        logger.info(f"部门 {dept_name} 删除成功")
        return DetailResponse(data=[], msg="删除成功")

    @action(methods=["GET", "POST"], detail=False, permission_classes=[IsAuthenticated])
    def dept_lazy_tree(self, request, *args, **kwargs):
        self.request.query_params.get("parent")
        is_superuser = request.user.is_superuser
        if is_superuser:
            queryset = Dept.objects.values("id", "name", "parent", "status", "node_type")
        else:
            data_range = request.user.role.values_list("data_range", flat=True)
            user_dept_id = request.user.dept.id
            dept_list = [user_dept_id]
            data_range_list = list(set(data_range))
            for item in data_range_list:
                if item in [0, 2]:
                    dept_list = [user_dept_id]
                elif item == 1:
                    dept_list = Dept.recursion_dept_info(dept_id=user_dept_id)
                elif item == 3:
                    dept_list = Dept.objects.values_list("id", flat=True)
                elif item == 4:
                    dept_list = request.user.role.values_list("dept", flat=True)
                else:
                    dept_list = []
            queryset = Dept.objects.filter(id__in=dept_list, status=True).values("id", "name", "parent", "node_type")
        return DetailResponse(data=queryset, msg="获取成功")

    @action(methods=["GET"], detail=False, permission_classes=[AnonymousUserPermission])
    def all_dept(self, request, *args, **kwargs):
        data = self.get_queryset().filter(status=True).order_by("sort").values("name", "id", "parent", "node_type")
        return DetailResponse(data=data, msg="获取成功")

    @action(methods=["GET"], detail=False, permission_classes=[OpenApiPermission], authentication_classes=[])
    def open_dept_info(self, request, *args, **kwargs):
        """
        第三方API同步部门接口
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        self.extra_filter_backends = []
        queryset = Dept.objects.all().order_by("id")
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True, request=request)
            return self.get_paginated_response(serializer.data)
        serializer = self.get_serializer(queryset, many=True, request=request)
        return SuccessResponse(data=serializer.data, msg="获取成功")

    @action(methods=["GET"], detail=False, permission_classes=[OpenApiPermission], authentication_classes=[])
    def open_single_dept_info(self, request, *args, **kwargs):
        """
        查询单个部门信息
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        nid = request.GET.get("nid", 0)
        if not nid:
            return ErrorResponse(msg="参数不合法")
        instance = Dept.objects.get(id=nid)
        serializer = self.get_serializer(instance)
        serializer_data = self.set_mask_fields(serializer)
        return DetailResponse(data=serializer_data, msg="获取成功")

    @action(methods=["POST"], detail=False, permission_classes=[OpenApiPermission], authentication_classes=[])
    def open_create_dept_info(self, request, *args, **kwargs):
        """
        第三方API创建部门信息
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        return self.create(request, *args, **kwargs)

    @action(methods=["PUT"], detail=False, permission_classes=[OpenApiPermission], authentication_classes=[])
    def open_update_dept_info(self, request, *args, **kwargs):
        """
        第三方API修改部门信息
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        partial = request.GET.get("partial", False)
        nid = request.GET.get("nid", 0)
        if not nid:
            return ErrorResponse(msg="参数不合法")
        instance = Dept.objects.get(id=nid)
        serializer = self.get_serializer(instance, data=request.data, request=request, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        if getattr(instance, "_prefetched_objects_cache", None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}
        return DetailResponse(data=serializer.data, msg="更新成功")

    @action(methods=["GET"], detail=False, permission_classes=[AnonymousUserPermission])
    def all_dept_not_extra(self, request, *args, **kwargs):
        self.extra_filter_backends = []
        queryset = self.filter_queryset(self.get_queryset())
        data = queryset.filter(status=True).order_by("sort").values("name", "id", "parent", "node_type", "status")
        return DetailResponse(data=data, msg="获取成功")

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def filter_dept_or_user(self, request):
        k = str(request.GET.get("k", "")).upper()
        if not k:
            return ErrorResponse(msg="参数不合法")
        result = []
        dept_result = Dept.objects.filter(Q(name__icontains=k) | Q(short_name__icontains=k))
        users_result = Users.objects.filter(Q(username__icontains=k) | Q(name__icontains=k), is_active=True)
        for item in dept_result:
            result.append(
                {
                    "id": item.id,
                    "type": "dept",
                    "name": item.name,
                    "key": item.key,
                    "short_name": item.short_name,
                    "node_type": item.node_type,
                }
            )
        for item in users_result:
            result.append(
                {
                    "id": item.id,
                    "type": "user",
                    "name": item.name,
                    "key": item.username,
                    "alias": item.alias,
                    "node_type": item.node_type,
                }
            )
        return DetailResponse(result)

    @action(methods=["POST"], detail=False, permission_classes=[IsAdminUser])
    def import_department(self, request, *args, **kwargs):
        """
        根据Excel文件 导入组织架构
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        excel_data_info = request.FILES.get("file", None)
        if not excel_data_info:
            return ErrorResponse(msg="参数不合法")
        excel_data_info = (
            pd.read_excel(excel_data_info, dtype=str).replace(np.nan, "", regex=True).to_dict(orient="records")
        )

        def create_or_update_department(path, short_name, sort, phone, email, parent=None):
            parts = path.split("/")
            if not parts:
                return None
            current_name = str(parts[0]).strip()
            if not current_name:
                return None
            # 查找或创建当前部门
            department, created = Dept.objects.get_or_create(
                name=current_name,
                parent=parent,
                defaults={
                    "dept_full_path": f"{parent.dept_full_path}/{current_name}" if parent else current_name,
                    "short_name": short_name if len(parts) == 1 else "",
                    "sort": sort if len(parts) == 1 else 1,
                    "phone": phone if len(parts) == 1 else None,
                    "email": email if len(parts) == 1 else None,
                },
            )
            # 递归处理下一级
            if len(parts) > 1:
                create_or_update_department("/".join(parts[1:]), short_name, sort, phone, email, department)
            return department

        for item in excel_data_info:
            full_name = item.get("部门名称", "")
            str(item.get("部门编码", "")).strip()
            short_name = str(item.get("部门简称", "")).strip()
            sort = str(item.get("排序", "")).strip()
            phone = str(item.get("座机", "")).strip()
            email = str(item.get("邮箱", "")).strip()
            create_or_update_department(path=full_name, short_name=short_name, sort=sort, phone=phone, email=email)
        return DetailResponse(msg="导入成功")



"""登录、登出、验证码、Token 刷新等接口单元测试。"""

import pytest
from rest_framework import status

from tests.assertions import assert_unauthorized

pytestmark = [pytest.mark.django_db]


def test_captcha_get_200(api_client):
    """GET /base/api/system/captcha/ 返回 200 与 key、image_base（需 DB 可用）。"""
    response = api_client.get("/base/api/system/captcha/")
    assert response.status_code == status.HTTP_200_OK
    data = response.json()
    assert data.get("code") == 2000, data
    assert "data" in data
    assert "key" in data["data"]
    assert "image_base" in data["data"]


def test_login_post_empty_400(api_client):
    """POST login 无 body 或缺少字段返回 400。"""
    response = api_client.post("/base/api/system/login/", {}, format="json")
    assert response.status_code in (status.HTTP_400_BAD_REQUEST, status.HTTP_401_UNAUTHORIZED)


def test_login_post_invalid_credentials_401(api_client):
    """POST login 错误账号密码返回 401/400。"""
    response = api_client.post(
        "/base/api/system/login/",
        {"username": "nonexistent", "password": "wrong"},
        format="json",
    )
    assert response.status_code in (status.HTTP_400_BAD_REQUEST, status.HTTP_401_UNAUTHORIZED)


def test_login_post_success_200(api_client, test_user):
    """POST login 正确账号密码返回 200 与 access/refresh。"""
    response = api_client.post(
        "/base/api/system/login/",
        {"username": test_user.username, "password": "TestPassword123"},
        format="json",
    )
    if response.status_code == 200:
        data = response.json()
        assert data.get("code") == 2000
        assert "data" in data and "access" in data["data"]


def test_logout_post_requires_auth(api_client):
    """POST logout 未认证返回 401 或 200+错误码。"""
    response = api_client.post("/base/api/system/logout/", {}, format="json")
    assert_unauthorized(response)


def test_logout_post_ok(authenticated_client):
    """POST logout 已认证返回 200。"""
    response = authenticated_client.post("/base/api/system/logout/", {}, format="json")
    assert response.status_code == status.HTTP_200_OK


def test_token_refresh_post_empty_400(api_client):
    """POST token/refresh/ 无 refresh 返回 400。"""
    response = api_client.post("/base/api/system/token/refresh/", {}, format="json")
    assert response.status_code in (status.HTTP_400_BAD_REQUEST, status.HTTP_401_UNAUTHORIZED)


def test_init_dictionary_get_200(api_client):
    """GET init/dictionary/ 可访问。"""
    response = api_client.get("/base/api/system/init/dictionary/")
    assert response.status_code == status.HTTP_200_OK


def test_init_settings_get_200(api_client):
    """GET init/settings/ 可访问。"""
    response = api_client.get("/base/api/system/init/settings/")
    assert response.status_code == status.HTTP_200_OK



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : file_list.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 文件上传、列表、Office 解析、UEditor 等。
import base64
import datetime
import hashlib
import io
import json
import logging
import os
import random
import re
import uuid
import zipfile
import xml.etree.ElementTree as ET
from urllib.parse import quote

from django.conf import settings
from django.contrib.auth.models import AnonymousUser
from django.core.files.uploadedfile import InMemoryUploadedFile
from django.http import HttpResponse, StreamingHttpResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework import serializers
from rest_framework.decorators import action
from rest_framework.permissions import (
    AllowAny,
    IsAuthenticated,  # 判断用户是否登录
)

from django_base_ai import dispatch
from django_base_ai.system.models import ChunkUploadSession, FileList
from django_base_ai.system.views.ueditor_settings import ueditor_settings, ueditor_upload_settings
from django_base_ai.utils.json_response import DetailResponse, ErrorResponse
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.string_util import format_bytes
from django_base_ai.utils.validator import CustomValidationError
from django_base_ai.utils.office_extract import (
    extract_docx_text as extract_docx_text_impl,
    extract_pptx_text as extract_pptx_text_impl,
    extract_xlsx_text as extract_xlsx_text_impl,
    get_file_stream_from_request,
)
from django_base_ai.utils.viewset import CustomModelViewSet
from django_base_ai.utils.xss_cleaned import cleaned_html_file

logger = logging.getLogger(__name__)


def _safe_file_url(instance):
    """将 file_url 转为对外安全的 Web 路径，不暴露物理路径。"""
    if not instance.file_url and not getattr(instance, "url", None):
        return ""
    # 已是完整 http(s) 的保持原样
    raw = (instance.file_url or "").strip()
    if raw.startswith("http://") or raw.startswith("https://"):
        return raw
    # 优先用 FileField url 得到相对路径，再拼成 /media/xxx
    if getattr(instance, "url", None) and instance.url:
        path = f"{settings.MEDIA_URL}{instance.url}".replace("//", "/")
        return f"/{path}" if not path.startswith("/") else path
    # 从物理路径中截取 media 及之后部分，返回 /media/...
    if "media" in raw:
        after_media = raw.split("media", 1)[1].replace("\\", "/").strip("/")
        return f"/media/{after_media}"
    return raw if raw.startswith("/") else f"/{raw}"


class FileSerializer(CustomModelSerializer):
    url = serializers.SerializerMethodField(read_only=True)
    file_url = serializers.SerializerMethodField(read_only=True)

    def get_url(self, instance):
        if self.request.query_params.get("prefix"):
            if instance.file_url:
                return instance.file_url if instance.file_url.startswith("http") else f"/{instance.file_url}"
            return f"/{settings.MEDIA_URL}/{str(instance.url)}"
        return f"{instance.url.url}" or (f"{settings.MEDIA_URL}/{str(instance.url)}")

    def get_file_url(self, instance):
        return _safe_file_url(instance)

    def validate(self, attrs):
        get_dictionary_config = dispatch.get_dictionary_config()
        ext_str, ext = "", []
        for item in get_dictionary_config.get("file_type", {}).get("children", []):
            ext_str += item.get("value", "").lower() + ","
        ext = ext_str.split(",")
        print("ext==", ext)
        file = self.initial_data.get("file")
        fn, t = os.path.splitext(file.name)
        if t[1:].lower() not in ext:
            raise CustomValidationError("文件格式不合法")
        return attrs

    def create(self, validated_data):
        file_engine = dispatch.get_system_config_values("file_storage.file_engine") or "local"
        file_backup = dispatch.get_system_config_values("file_storage.file_backup")
        file = self.initial_data.get("file")
        file_size = file.size
        file.name = (
            file.name if len(file.name.split(".")) > 1 else str(uuid.uuid4()) + "." + file.content_type.split("/")[1]
        )

        # 检查是否为 HTML/HTM 文件
        file_extension = os.path.splitext(file.name)[1].lower()
        is_html_file = file_extension in [".html", ".htm"]

        if is_html_file:
            # 对于 HTML 文件，需要清理 XSS
            from django.core.files.base import ContentFile

            from django_base_ai.utils.xss_cleaned import cleaned_html_file

            # 保存原始文件的content_type
            original_content_type = file.content_type

            # 读取文件内容
            file_content = b""
            for chunk in file.chunks():
                file_content += chunk

            try:
                # 解码为字符串
                html_content = file_content.decode("utf-8")
                # 清理 XSS
                cleaned_content = cleaned_html_file(html_content)
                # 重新编码为字节
                cleaned_bytes = cleaned_content.encode("utf-8")

                # 创建清理后的文件对象
                cleaned_file = ContentFile(cleaned_bytes, name=file.name)
                # 为ContentFile添加content_type属性
                cleaned_file.content_type = original_content_type
                file = cleaned_file
                file_size = len(cleaned_bytes)
            except Exception:
                # 如果清理失败，使用原文件
                pass

        validated_data["name"] = file.name
        validated_data["size"] = file_size
        md5 = hashlib.md5()
        for chunk in file.chunks():
            md5.update(chunk)
        validated_data["md5sum"] = md5.hexdigest()
        validated_data["engine"] = file_engine
        validated_data["mime_type"] = file.content_type
        if file_backup:
            validated_data["url"] = file
        validated_data["url"] = file
        # 审计字段
        if self.request and self.request.user and not isinstance(self.request.user, AnonymousUser):
            try:
                request_user = self.request.user
                if hasattr(request_user, "dept") and request_user.dept:
                    validated_data["dept_belong_id"] = request_user.dept.id
                validated_data["creator"] = request_user.id
                validated_data["modifier"] = request_user.id
            except Exception as e:
                logger.warning(f"设置文件审计字段时出错: {e}")
        return super().create(validated_data)

    class Meta:
        model = FileList
        fields = "__all__"


class FileListInitSerializer(CustomModelSerializer):
    def save(self, **kwargs):
        instance = super().save(**kwargs)
        file_engine = dispatch.get_system_config_values("file_storage.file_engine") or "local"
        dispatch.get_system_config_values("file_storage.file_backup")
        file_url = f"/{settings.MEDIA_ROOT}{self.instance.file_url}"
        local_file_path = f"{settings.BASE_DIR}{file_url}"
        print("local_file_path=", local_file_path)
        with open(local_file_path, "br") as file:
            file_bytes_io = io.BytesIO(file.read())
            file_size = os.path.getsize(local_file_path)
            print("file_size==", file_size)
            instance.size = file_size
            instance.engine = file_engine
            instance.file_url = file_url
            instance.url = InMemoryUploadedFile(file_bytes_io, None, instance.name, None, file_size, None)
            instance.save()
            file_bytes_io.close()
        return instance

    class Meta:
        model = FileList
        fields = "__all__"


def file_iterator(filename, chunk_size=512):
    """read file"""
    with open(filename, "rb") as f:
        while True:
            c = f.read(chunk_size)
            if c:
                yield c
            else:
                break


rstr = r"[\/\\\:\*\?\"\<\>\|]"

# 断点续传：切片临时目录（相对 MEDIA_ROOT）
CHUNK_UPLOAD_DIR = "chunk_upload"


class FileViewSet(CustomModelViewSet):
    """
    文件管理接口
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = FileList.objects.all()
    serializer_class = FileSerializer
    filter_fields = ["name"]
    search_fields = ["name"]

    @csrf_exempt
    @action(methods=["GET", "POST"], detail=False, permission_classes=[])
    def ueditor(self, request):
        action = self.request.query_params.get("action", "")
        reponse_action = {
            "config": self.get_ueditor_settings,
            "uploadimage": self.upload_file,
            "uploadscrawl": self.upload_file,
            "uploadvideo": self.upload_file,
            "uploadfile": self.upload_file,
        }
        return reponse_action[action](request)

    def get_ueditor_settings(self, request):
        return HttpResponse(
            json.dumps(ueditor_upload_settings, ensure_ascii=False), content_type="application/javascript"
        )

    # 保存上传的文件
    def save_upload_file(self, file, file_path):
        # 检查是否为 HTML/HTM 文件
        file_extension = os.path.splitext(file_path)[1].lower()
        is_html_file = file_extension in [".html", ".htm"]

        if is_html_file:
            # 对于 HTML 文件，需要清理 XSS
            from django_base_ai.utils.xss_cleaned import cleaned_html_file

            # 读取文件内容
            file_content = b""
            for chunk in file.chunks():
                file_content += chunk

            try:
                # 解码为字符串
                html_content = file_content.decode("utf-8")
                # 清理 XSS
                cleaned_content = cleaned_html_file(html_content)
                # 重新编码为字节
                cleaned_bytes = cleaned_content.encode("utf-8")

                # 写入清理后的内容
                with open(file_path, "wb") as f:
                    f.write(cleaned_bytes)
            except Exception as e:
                return f"清理HTML文件XSS错误:{e}"
        else:
            # 非 HTML 文件，直接保存
            with open(file_path, "wb") as f:
                try:
                    for chunk in file.chunks():
                        f.write(chunk)
                except Exception as e:
                    return f"写入文件错误:{e}"

        return "SUCCESS"

    def get_path_format_vars(self):
        return {
            "year": datetime.datetime.now().strftime("%Y"),
            "month": datetime.datetime.now().strftime("%m"),
            "day": datetime.datetime.now().strftime("%d"),
            "date": datetime.datetime.now().strftime("%Y%m%d"),
            "time": datetime.datetime.now().strftime("%H%M%S"),
            "datetime": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
            "rnd": random.randrange(100, 999),
        }

    def get_output_path(self, path_format_var):
        """
        取得输出文件的路径
        :param path_format_var:
        :return:
        """
        file_name = (ueditor_settings["defaultPathFormat"] % path_format_var).replace("\\", "/")
        # 分解OutputPathFormat
        output_path = os.path.join(f"{settings.MEDIA_ROOT}", "ueditor", f"{self.request.user.id}")
        if not os.path.exists(output_path):
            os.makedirs(output_path)
        return (file_name, output_path)

    # 涂鸦功能上传处理
    def save_scrawl_file(self, request, file_path, file_name):

        instance = None
        try:
            content = request.data.get(ueditor_upload_settings.get("scrawlFieldName", "upfile"))
            f = open(os.path.join(settings.BASE_DIR, file_path, file_name), "wb")
            f.write(base64.b64decode(content))
            f.close()
            state = "SUCCESS"
            instance = FileList.save_file(request, file_path, file_name, mime_type="image/png")
        except Exception as e:
            state = f"写入图片文件错误:{e}"
        return state, instance

    def upload_file(self, request):
        """上传文件"""
        state = "SUCCESS"
        action = self.request.query_params.get("action")
        # 上传文件
        upload_field_name_list = {
            "uploadfile": "fileFieldName",
            "uploadimage": "imageFieldName",
            "uploadscrawl": "scrawlFieldName",
            "catchimage": "catcherFieldName",
            "uploadvideo": "videoFieldName",
        }
        upload_field_name = self.request.query_params.get(
            upload_field_name_list[action], ueditor_upload_settings.get(action, "upfile")
        )
        # 上传涂鸦，涂鸦是采用base64编码上传的，需要单独处理
        if action == "uploadscrawl":
            upload_file_name = "scrawl.png"
            upload_file_size = 0
        else:
            # 取得上传的文件
            file = request.FILES.get(upload_field_name, None)
            if file is None:
                return HttpResponse(json.dumps("{'state:'ERROR'}"), content_type="application/javascript")
            upload_file_name = file.name
            upload_file_size = file.size

        # 取得上传的文件的原始名称
        upload_original_name, upload_original_ext = os.path.splitext(upload_file_name)
        # 文件类型检验
        upload_allow_type = {
            "uploadfile": "fileAllowFiles",
            "uploadimage": "imageAllowFiles",
            "uploadvideo": "videoAllowFiles",
        }
        if action in upload_allow_type:
            allow_type = list(
                self.request.query_params.get(
                    upload_allow_type[action], ueditor_upload_settings.get(upload_allow_type[action], "")
                )
            )
            if upload_original_ext.lower() not in allow_type:
                state = f"服务器不允许上传{upload_original_ext}类型的文件。"
                return HttpResponse({"state": state}, content_type="application/javascript")

        # 大小检验
        upload_max_size = {
            "uploadfile": "filwMaxSize",
            "uploadimage": "imageMaxSize",
            "uploadscrawl": "scrawlMaxSize",
            "uploadvideo": "videoMaxSize",
        }
        max_size = int(
            self.request.query_params.get(
                upload_max_size[action], ueditor_upload_settings.get(upload_max_size[action], 0)
            )
        )
        if max_size != 0:
            if upload_file_size > max_size:
                state = f"上传文件大小不允许超过{format_bytes(max_size)}。"
                return HttpResponse({"state": state}, content_type="application/javascript")

        path_format_var = self.get_path_format_vars()
        path_format_var.update(
            {
                "basename": upload_original_name,
                "extname": upload_original_ext[1:],
                "filename": upload_file_name,
            }
        )
        # 取得输出文件的路径
        format_file_name, output_path = self.get_output_path(path_format_var)
        # 所有检测完成后写入文件
        file_instance = None
        if state == "SUCCESS":
            if action == "uploadscrawl":
                state, file_instance = self.save_scrawl_file(request, file_path=output_path, file_name=format_file_name)
            else:
                file = request.FILES.get(upload_field_name, None)
                # 保存到文件中，如果保存错误，需要返回ERROR
                state = self.save_upload_file(file, os.path.join(settings.BASE_DIR, output_path, format_file_name))
                # 保存到附件管理中
                file_instance = FileList.save_file(request, output_path, format_file_name, mime_type=file.content_type)

        # 返回数据
        return_info = {
            "url": file_instance.file_url
            if file_instance
            else os.path.join(output_path, format_file_name),  # 保存后的文件名称
            "original": upload_file_name,  # 原始文件名
            "type": upload_original_ext,
            "state": state,  # 上传状态，成功时返回SUCCESS,其他任何值将原样返回至图片上传框中
            "size": upload_file_size,
        }
        return HttpResponse(json.dumps(return_info, ensure_ascii=False), content_type="application/javascript")

    @action(methods=["GET"], detail=False)
    def download_file(self, request):
        """
        下载文件 需要验证用户权限
        """
        fid = request.GET.get("fid", 0)
        if not fid:
            return ErrorResponse(msg="参数错误请重试")
        file_list = FileList.objects.filter(id=fid).first()
        if not file_list:
            return ErrorResponse(msg="参数错误请重试")
        str(os.path.splitext(file_list.name)[-1]).upper()
        file_path = f"{settings.BASE_DIR}{file_list.file_url}"
        print("file_path=", file_path)
        response = StreamingHttpResponse(file_iterator(file_path))
        response["Content-Type"] = "application/octet-stream"
        response["Content-Disposition"] = 'attachment;filename="{}"'.format(
            quote(re.sub(rstr, "_", file_list.name)),
        )
        return response

    @csrf_exempt
    @action(methods=["post"], detail=False, permission_classes=[AllowAny], authentication_classes=[])
    def upload_allow_file(self, request, *args, **kwargs):
        fid = request.data.get("fid", "")
        file_names = []
        file_list_to_insert = list()
        files = request.FILES.getlist("file", [])
        get_dictionary_config = dispatch.get_dictionary_config()
        ext = (
            ",".join([item.get("value") for item in get_dictionary_config.get("file_type", {}).get("children")])
            .lower()
            .split(",")
        )
        for item in files:
            fn, t = os.path.splitext(item.name)
            if t[1:].lower() in ext:
                directory_path = os.path.join(f"{settings.BASE_DIR}/{settings.MEDIA_ROOT}", fid)
                if not os.path.exists(directory_path):
                    os.makedirs(directory_path)
                filename = f"{uuid.uuid4().hex}{t.lower()}"
                file_path = directory_path + "/" + filename
                md5 = hashlib.md5()
                with open(file_path, "wb") as f:
                    for c in item.chunks():
                        f.write(c)
                        md5.update(c)
                    f.close()
                file_names.append(f"{settings.MEDIA_URL}{filename}")
                file_list_to_insert.append(
                    FileList(
                        name=item.name,
                        file_url=f"{settings.MEDIA_URL}{filename}",
                        mime_type=item.content_type,
                        size=item.size,
                        md5sum=md5.hexdigest(),
                    )
                )
            if len(file_list_to_insert) > 0:
                FileList.objects.bulk_create(file_list_to_insert)
        return DetailResponse(file_names)

    # ---------- 断点续传 ----------

    @action(methods=["POST"], detail=False, permission_classes=[IsAuthenticated])
    def check_file_by_md5(self, request, *args, **kwargs):
        """
        步骤1：前端传文件 MD5，后端判断是否已存在（秒传）。
        请求体: md5（必填）, filename/name、total_size、chunk_total（可选）
        响应: exists=True 返回已有 file 信息；exists=False 创建上传会话并返回 upload_id。
        """
        data = request.data
        file_md5 = data.get("md5","").strip().lower()
        if not file_md5 or len(file_md5) != 32:
            return ErrorResponse(msg="请提供有效的 md5（32 位）")
        existing = FileList.objects.filter(md5sum=file_md5).first()
        if existing:
            file_url = existing.file_url or (
                f"/{settings.MEDIA_URL}{existing.url}".replace("//", "/") if existing.url else ""
            )
            return DetailResponse(
                data={
                    "exists": True,
                    "file_id": existing.id,
                    "file_url": file_url,
                    "name": existing.name,
                    "size": existing.size,
                },
                msg="文件已存在，可秒传",
            )
        upload_id = uuid.uuid4().hex
        filename = (data.get("filename") or data.get("name") or "").strip()
        try:
            total_size = int(data.get("total_size") or 0)
        except (TypeError, ValueError):
            total_size = 0
        try:
            chunk_total = int(data.get("chunk_total") or 0)
        except (TypeError, ValueError):
            chunk_total = 0
        session = ChunkUploadSession(
            upload_id=upload_id,
            file_md5=file_md5,
            filename=filename,
            total_size=total_size,
            chunk_total=chunk_total,
            uploaded_chunk_indices=[],
        )
        if request.user and not isinstance(request.user, AnonymousUser):
            try:
                session.creator = str(request.user.id)
                session.modifier = str(request.user.id)
                if getattr(request.user, "dept_id", None):
                    session.dept_belong_id = str(request.user.dept_id)
            except Exception as e:
                logger.warning("设置审计字段时出错: %s", e)
        session.save()
        return DetailResponse(
            data={"exists": False, "upload_id": upload_id},
            msg="文件不存在，请上传切片",
        )

    @action(methods=["POST"], detail=False, permission_classes=[IsAuthenticated])
    def upload_chunk(self, request, *args, **kwargs):
        """
        步骤2：上传单个切片。请求 multipart：upload_id、chunk_index（从 0 开始）、chunk_total、file 或 chunk。
        同一 chunk_index 重复上传会覆盖文件，会话中只记录一次（幂等）。
        """
        data = request.data
        upload_id = data.get("upload_id","").strip()
        if not upload_id:
            return ErrorResponse(msg="缺少 upload_id")
        if not re.match(r"^[a-zA-Z0-9\-]{1,64}$", upload_id):
            return ErrorResponse(msg="upload_id 仅允许字母、数字、短横线，长度 1~64")
        try:
            chunk_index = int(data.get("chunk_index"))
            chunk_total = int(data.get("chunk_total"))
        except (TypeError, ValueError):
            return ErrorResponse(msg="chunk_index / chunk_total 必须为整数")
        if chunk_index < 0 or chunk_total < 1 or chunk_index >= chunk_total:
            return ErrorResponse(msg="chunk_index 或 chunk_total 不合法")
        chunk_file = request.FILES.get("file") or request.FILES.get("chunk")
        if not chunk_file:
            return ErrorResponse(msg="请上传切片文件 file 或 chunk")
        chunk_dir = os.path.join(settings.BASE_DIR, settings.MEDIA_ROOT, CHUNK_UPLOAD_DIR, upload_id)
        try:
            os.makedirs(chunk_dir, exist_ok=True)
        except OSError as e:
            logger.exception("创建切片目录失败: %s", e)
            return ErrorResponse(msg="创建切片目录失败")
        chunk_path = os.path.join(chunk_dir, str(chunk_index))
        try:
            with open(chunk_path, "wb") as f:
                for chunk in chunk_file.chunks():
                    f.write(chunk)
        except OSError as e:
            logger.exception("写入切片失败: %s", e)
            return ErrorResponse(msg="写入切片失败")
        session = ChunkUploadSession.objects.filter(upload_id=upload_id).first()
        uploaded_count = 1
        if session:
            indices = sorted(set((session.uploaded_chunk_indices or []) + [chunk_index]))
            session.uploaded_chunk_indices = indices
            if session.chunk_total == 0:
                session.chunk_total = chunk_total
            session.save(update_fields=["uploaded_chunk_indices", "chunk_total", "update_datetime"])
            uploaded_count = len(indices)
        return DetailResponse(
            data={"chunk_index": chunk_index, "chunk_total": chunk_total, "uploaded_count": uploaded_count},
            msg="切片上传成功",
        )

    @action(methods=["POST"], detail=False, permission_classes=[IsAuthenticated])
    def get_uploaded_chunks(self, request, *args, **kwargs):
        """
        查询某次上传已完成的切片序号，用于断点续传（前端只传缺失的切片）。
        请求: upload_id（query 或 body）
        响应: uploaded_chunk_indices, chunk_total
        """
        upload_id = (request.data.get("upload_id") or request.query_params.get("upload_id") or "").strip()
        if not upload_id:
            return ErrorResponse(msg="请提供 upload_id")
        session = ChunkUploadSession.objects.filter(upload_id=upload_id).first()
        if session:
            return DetailResponse(
                data={
                    "uploaded_chunk_indices": list(session.uploaded_chunk_indices or []),
                    "chunk_total": session.chunk_total,
                },
                msg="ok",
            )
        chunk_dir = os.path.join(settings.BASE_DIR, settings.MEDIA_ROOT, CHUNK_UPLOAD_DIR, upload_id)
        if os.path.isdir(chunk_dir):
            indices = sorted([int(p) for p in os.listdir(chunk_dir) if p.isdigit()])
            return DetailResponse(
                data={"uploaded_chunk_indices": indices, "chunk_total": 0},
                msg="会话未入库，已从磁盘读取已上传切片",
            )
        return DetailResponse(
            data={"uploaded_chunk_indices": [], "chunk_total": 0},
            msg="未找到该 upload_id 的会话或切片目录",
        )

    def _cleanup_chunk_upload(self, upload_id, chunk_dir, chunk_indices, remove_final_path=None):
        """删除切片目录下文件、目录及该 upload_id 的会话记录；可选删除合并后的文件。"""
        if remove_final_path and os.path.isfile(remove_final_path):
            try:
                os.remove(remove_final_path)
            except OSError:
                pass
        for i in chunk_indices:
            try:
                os.remove(os.path.join(chunk_dir, str(i)))
            except OSError:
                pass
        try:
            os.rmdir(chunk_dir)
        except OSError:
            pass
        ChunkUploadSession.objects.filter(upload_id=upload_id).delete()

    @action(methods=["POST"], detail=False, permission_classes=[IsAuthenticated])
    def merge_chunks(self, request, *args, **kwargs):
        """
        步骤3：合并切片，校验整体 MD5 与可选 total_size，通过则入库并清理临时目录。
        请求体: upload_id, md5, filename 或 name；可选 total_size, mime_type 或 content_type
        """
        data = request.data
        upload_id = data.get("upload_id","").strip()
        file_md5 = data.get("md5","").strip().lower()
        filename = data.get("name","").strip()
        mime_type = data.get("mime_type","application/octet-stream").strip()
        if not upload_id or not file_md5 or len(file_md5) != 32:
            return ErrorResponse(msg="请提供 upload_id 与有效的 md5（32 位）")
        if not filename:
            return ErrorResponse(msg="请提供 filename")
        try:
            total_size = int(data.get("total_size",0))
        except (TypeError, ValueError):
            total_size = 0
        chunk_dir = os.path.join(settings.BASE_DIR, settings.MEDIA_ROOT, CHUNK_UPLOAD_DIR, upload_id)
        if not os.path.isdir(chunk_dir):
            return ErrorResponse(msg="未找到该 upload_id 的切片目录，请先上传切片")
        chunk_indices = sorted([int(p) for p in os.listdir(chunk_dir) if p.isdigit()])
        if not chunk_indices:
            return ErrorResponse(msg="切片目录为空")
        ext = os.path.splitext(filename)[1].lower() or ""
        files_subdir = os.path.join(settings.BASE_DIR, settings.MEDIA_ROOT, "files")
        final_name = file_md5 + ext
        final_path = os.path.join(files_subdir, final_name)
        try:
            os.makedirs(files_subdir, exist_ok=True)
        except OSError as e:
            logger.exception("创建 files 目录失败: %s", e)
            return ErrorResponse(msg="创建文件目录失败")
        md5_calc = hashlib.md5()
        size_sum = 0
        try:
            with open(final_path, "wb") as out:
                for i in chunk_indices:
                    chunk_path = os.path.join(chunk_dir, str(i))
                    if not os.path.isfile(chunk_path):
                        raise ValueError(f"缺少切片 {i}")
                    with open(chunk_path, "rb") as inc:
                        buf = inc.read()
                    out.write(buf)
                    md5_calc.update(buf)
                    size_sum += len(buf)
        except (OSError, ValueError) as e:
            self._cleanup_chunk_upload(upload_id, chunk_dir, chunk_indices, final_path)
            return ErrorResponse(msg=f"合并失败: {e}")
        merged_md5 = md5_calc.hexdigest().lower()
        if merged_md5 != file_md5:
            self._cleanup_chunk_upload(upload_id, chunk_dir, chunk_indices, final_path)
            return ErrorResponse(msg=f"文件 MD5 校验失败：期望 {file_md5}，实际 {merged_md5}")
        if total_size and size_sum != total_size:
            self._cleanup_chunk_upload(upload_id, chunk_dir, chunk_indices, final_path)
            return ErrorResponse(msg=f"文件大小不一致：期望 {total_size}，实际 {size_sum}")
        file_url = f"{settings.MEDIA_URL.rstrip('/')}/files/{final_name}".replace("//", "/")
        rel_path = f'files/{final_name}'
        instance = FileList(
            name=filename,
            file_url=file_url,
            url=rel_path,
            md5sum=merged_md5,
            size=size_sum,
            mime_type=mime_type,
            engine=dispatch.get_system_config_values("file_storage.file_engine") or "local",
        )
        if request.user and not isinstance(request.user, AnonymousUser):
            try:
                instance.creator = str(request.user.id)
                instance.modifier = str(request.user.id)
                if getattr(request.user, "dept_id", None):
                    instance.dept_belong_id = str(request.user.dept_id)
            except Exception as e:
                logger.warning("设置审计字段时出错: %s", e)
        instance.save()
        self._cleanup_chunk_upload(upload_id, chunk_dir, chunk_indices)
        return DetailResponse(
            data={
                "file_id": instance.id,
                "file_url": instance.file_url,
                "name": instance.name,
                "size": instance.size,
                "md5sum": instance.md5sum,
            },
            msg="合并并校验成功",
        )

    # ---------- Office 文件解压提取明文（docx/xlsx/pptx 本质为 ZIP+XML），用于提供给 AI ----------

    @action(methods=["GET", "POST"], detail=False, permission_classes=[IsAuthenticated])
    def extract_docx_text(self, request, *args, **kwargs):
        """
        解压 docx（ZIP+XML）并返回正文明文。
        请求: file_id（query/body）或 multipart 上传 file。
        响应: text 为提取的纯文本。
        """
        file_like, name = get_file_stream_from_request(request)
        if file_like is None:
            return ErrorResponse(msg="请提供 file_id 或上传 file")
        try:
            text = extract_docx_text_impl(file_like)
        except (zipfile.BadZipFile, ET.ParseError, KeyError) as e:
            logger.exception("extract_docx_text 失败: %s", e)
            return ErrorResponse(msg=f"文件不是有效的 docx 或解析失败: {e}")
        return DetailResponse(data={"text": text, "filename": name}, msg="ok")

    @action(methods=["GET", "POST"], detail=False, permission_classes=[IsAuthenticated])
    def extract_xlsx_text(self, request, *args, **kwargs):
        """
        解压 xlsx（ZIP+XML）并返回单元格等文本信息（明文）。
        请求: file_id 或上传 file。
        """
        file_like, name = get_file_stream_from_request(request)
        if file_like is None:
            return ErrorResponse(msg="请提供 file_id 或上传 file")
        try:
            text = extract_xlsx_text_impl(file_like)
        except (zipfile.BadZipFile, ET.ParseError, KeyError) as e:
            logger.exception("extract_xlsx_text 失败: %s", e)
            return ErrorResponse(msg=f"文件不是有效的 xlsx 或解析失败: {e}")
        return DetailResponse(data={"text": text, "filename": name}, msg="ok")

    @action(methods=["GET", "POST"], detail=False, permission_classes=[IsAuthenticated])
    def extract_pptx_text(self, request, *args, **kwargs):
        """
        解压 pptx（ZIP+XML）并返回幻灯片内文本（明文）。
        请求: file_id 或上传 file。
        """
        file_like, name = get_file_stream_from_request(request)
        if file_like is None:
            return ErrorResponse(msg="请提供 file_id 或上传 file")
        try:
            text = extract_pptx_text_impl(file_like)
        except (zipfile.BadZipFile, ET.ParseError, KeyError) as e:
            logger.exception("extract_pptx_text 失败: %s", e)
            return ErrorResponse(msg=f"文件不是有效的 pptx 或解析失败: {e}")
        return DetailResponse(data={"text": text, "filename": name}, msg="ok")

    @action(methods=["POST"], detail=False, permission_classes=[IsAuthenticated])
    def html_cleaned_xss(self, request, *args, **kwargs):
        """
        清理HTML内容中的XSS攻击
        """
        # 获取HTML内容
        html_content = request.data.get("html", "")

        # 检查输入是否为空
        if not html_content:
            return ErrorResponse(msg="HTML内容不能为空")

        # 如果传入的是字节类型，需要先解码
        if isinstance(html_content, bytes):
            html_content = html_content.decode("utf-8")

        # 清理 XSS
        cleaned_content = cleaned_html_file(html_content)

        # 返回清理后的内容
        return DetailResponse(
            data={
                "cleaned_html": cleaned_content,
                "original_length": len(html_content),
                "cleaned_length": len(cleaned_content),
            },
            msg="操作成功",
        )



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : datav.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 数据可视化大屏配置与数据接口。
import datetime
import logging
import time

from django.db import connection
from django.db.models import Count, Sum
from django.db.models.functions import TruncDay
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated
from rest_framework.viewsets import GenericViewSet

from django_base_ai.system.models import FileList, LoginLog, Users
from django_base_ai.system.views.login_log import LoginLogSerializer
from django_base_ai.utils.json_response import DetailResponse
from django_base_ai.utils.string_util import format_bytes


def jx_timestamp():
    cur_time = datetime.datetime.now()
    a = datetime.datetime.strftime(cur_time, "%Y-%m-%d %H:%M:%S")
    timestamp = int(time.mktime(time.strptime(a, "%Y-%m-%d %H:%M:%S")))
    time_array = time.localtime(timestamp)
    other_style_time = time.strftime("%Y-%m-%d %H:%M:%S", time_array)
    return other_style_time


logger = logging.getLogger(__name__)


class DataVViewSet(GenericViewSet):
    queryset = LoginLog.objects.all()
    serializer_class = LoginLogSerializer
    # extra_filter_backends = []
    ordering_fields = ["create_datetime"]

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def users_login_total(self, request):
        """
        用户登录总数数据
        :param request:
        :return:
        """
        login_total = LoginLog.objects.all().count()
        return DetailResponse(data={"login_total": login_total}, msg="获取成功")

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def users_total(self, request):
        """
        用户总数
        :param request:
        :return:
        """
        users_total = Users.objects.all().count()
        return DetailResponse(
            data={
                "users_total": users_total,
            },
            msg="获取成功",
        )

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def attachment_total(self, request):
        """
        附件统计数据
        :param request:
        :return:
        """
        count = FileList.objects.all().count()
        if count != 0:
            data = FileList.objects.aggregate(sum_size=Sum("size"))
        else:
            data = {"sum_size": 0}
        return DetailResponse(
            data={"count": count, "occupy_space": format_bytes(data.get("sum_size") or 0)}, msg="获取成功"
        )

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def database_total(self, request):
        """
        数据库统计数据
        :param request:
        :return:
        """
        count = len(connection.introspection.table_names())
        database_type = connection.settings_dict["ENGINE"]
        sql = None
        if "mysql" in database_type:
            sql = "SELECT SUM(data_length + index_length) AS size FROM information_schema.TABLES WHERE table_schema = DATABASE()"
        elif "postgres" in database_type or "psqlextra" in database_type:
            sql = """SELECT SUM(pg_total_relation_size(quote_ident(schemaname) || '.' || quote_ident(tablename))) AS size FROM pg_tables WHERE schemaname = current_schema();"""
        elif "oracle" in database_type:
            sql = "SELECT SUM(bytes) AS size FROM user_segments"
        elif "microsoft" in database_type:
            sql = "SELECT SUM(size) * 8 AS size FROM sys.database_files"
        else:
            space = 0
        if sql:
            with connection.cursor() as cursor:
                try:
                    cursor.execute(sql)
                    result = cursor.fetchone()
                    space = result[0]
                except Exception as e:
                    print(e)
                    space = "无权限"
        return DetailResponse(data={"count": count, "space": format_bytes(space or 0)}, msg="获取成功")

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def registered_user(self, request):
        """
        用户注册趋势
        :param request:
        :return:
        """
        day = 30
        today = datetime.datetime.today()
        seven_days_ago = today - datetime.timedelta(days=day)
        users = (
            Users.objects.filter(create_datetime__gte=seven_days_ago)
            .annotate(day=TruncDay("create_datetime"))
            .values("day")
            .annotate(count=Count("id"))
            .order_by("-day")
        )
        result = []
        data_dict = {ele.get("day").strftime("%Y-%m-%d"): ele.get("count") for ele in users}
        for i in range(day):
            date = (today - datetime.timedelta(days=i)).strftime("%Y-%m-%d")
            result.append({"day": date, "count": data_dict[date] if date in data_dict else 0})
        result = sorted(result, key=lambda x: x["day"])
        return DetailResponse(data={"registered_user_list": result}, msg="获取成功")

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def login_user(self, request):
        """
        用户登录趋势
        :param request:
        :return:
        """
        day = 30
        today = datetime.datetime.today()
        seven_days_ago = today - datetime.timedelta(days=day)
        users = (
            LoginLog.objects.filter(create_datetime__gte=seven_days_ago)
            .annotate(day=TruncDay("create_datetime"))
            .values("day")
            .annotate(count=Count("id"))
            .order_by("-day")
        )
        result = []
        data_dict = {ele.get("day").strftime("%Y-%m-%d"): ele.get("count") for ele in users}
        for i in range(day):
            date = (today - datetime.timedelta(days=i)).strftime("%Y-%m-%d")
            result.append({"day": date, "count": data_dict[date] if date in data_dict else 0})
        result = sorted(result, key=lambda x: x["day"])
        return DetailResponse(data={"login_user": result}, msg="获取成功")

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def users_active(self, request):
        """
        用户新增活跃数据统计
        :param request:
        :return:
        """
        today = datetime.date.today()
        seven_days_ago = today - datetime.timedelta(days=6)
        thirty_days_ago = today - datetime.timedelta(days=29)

        today_users = Users.objects.filter(date_joined__date=today).count()
        today_logins = Users.objects.filter(last_login__date=today).count()
        three_days_users = Users.objects.filter(date_joined__gte=seven_days_ago).count()
        seven_days_users = Users.objects.filter(date_joined__gte=thirty_days_ago).count()
        seven_days_active = (
            Users.objects.filter(last_login__gte=seven_days_ago)
            .values("last_login")
            .annotate(count=Count("id", distinct=True))
            .count()
        )
        monthly_active = (
            Users.objects.filter(last_login__gte=thirty_days_ago)
            .values("last_login")
            .annotate(count=Count("id", distinct=True))
            .count()
        )

        data = {
            "today_users": today_users,
            "today_logins": today_logins,
            "three_days": three_days_users,
            "seven_days": seven_days_users,
            "seven_days_active": seven_days_active,
            "monthly_active": monthly_active,
        }
        return DetailResponse(data=data, msg="获取成功")



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : menu.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 菜单与路由树 CRUD、初始化。
import logging

from rest_framework import serializers
from rest_framework.decorators import action

from django_base_ai.system.models import Menu, MenuButton
from django_base_ai.system.views.menu_button import MenuButtonInitSerializer
from django_base_ai.utils.json_response import ErrorResponse, SuccessResponse
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class MenuSerializer(CustomModelSerializer):
    """
    菜单表的简单序列化器
    """

    menu_permission = serializers.SerializerMethodField(read_only=True)
    has_child = serializers.SerializerMethodField()

    def get_menu_permission(self, instance):
        queryset = instance.menuPermission.order_by("-name").values_list("name", flat=True)
        result = queryset if queryset else None
        logger.debug(f"菜单 {instance.name} 权限列表: {list(result) if result else []}")
        return result

    def get_has_child(self, instance):
        has_child = True if Menu.objects.filter(parent=instance.id).count() > 0 else False
        logger.debug(f"菜单 {instance.name} 是否有子菜单: {has_child}")
        return has_child

    class Meta:
        model = Menu
        fields = "__all__"
        read_only_fields = ["id"]


class MenuCreateSerializer(CustomModelSerializer):
    class Meta:
        model = Menu
        fields = "__all__"
        read_only_fields = ["id"]


class MenuInitSerializer(CustomModelSerializer):
    """
    递归深度获取数信息(用于生成初始化json文件)
    """

    name = serializers.CharField(required=False)
    children = serializers.SerializerMethodField()
    menu_button = serializers.SerializerMethodField()

    def get_children(self, obj: Menu):
        logger.debug(f"获取菜单 {obj.name} 的子菜单")
        data = []
        instance = Menu.objects.filter(parent_id=obj.id)
        if instance:
            serializer = MenuInitSerializer(instance=instance, many=True)
            data = serializer.data
        logger.debug(f"菜单 {obj.name} 子菜单数量: {len(data)}")
        return data

    def get_menu_button(self, obj: Menu):
        logger.debug(f"获取菜单 {obj.name} 的按钮权限")
        data = []
        instance = obj.menuPermission.order_by("method")
        if instance:
            data = list(instance.values("name", "value", "api", "method"))
        logger.debug(f"菜单 {obj.name} 按钮权限数量: {len(data)}")
        return data

    def save(self, **kwargs):
        logger.info(f"开始保存菜单初始化数据: {self.initial_data.get('name', '未知')}")
        instance = super().save(**kwargs)
        logger.info(f"菜单 {instance.name} 基础信息已保存，ID: {instance.id}")
        children = self.initial_data.get("children")
        menu_button = self.initial_data.get("menu_button")
        # 菜单表
        if children:
            for menu_data in children:
                menu_data["parent"] = instance.id
                filter_data = {
                    "name": menu_data["name"],
                    "path": menu_data["path"],
                    "component": menu_data["component"],
                }
                instance_obj = Menu.objects.filter(**filter_data).first()
                if instance_obj and not self.initial_data.get("reset"):
                    continue
                serializer = MenuInitSerializer(instance_obj, data=menu_data, request=self.request)
                serializer.is_valid(raise_exception=True)
                serializer.save()
                logger.info(f"子菜单 {menu_data['name']} 已保存")
        # 菜单按钮
        if menu_button:
            logger.info(f"保存菜单按钮，数量: {len(menu_button)}")
            for menu_button_data in menu_button:
                menu_button_data["menu"] = instance.id
                filter_data = {"menu": menu_button_data["menu"], "value": menu_button_data["value"]}
                instance_obj = MenuButton.objects.filter(**filter_data).first()
                serializer = MenuButtonInitSerializer(instance_obj, data=menu_button_data, request=self.request)
                serializer.is_valid(raise_exception=True)
                serializer.save()
        logger.info(f"菜单 {instance.name} 初始化完成")
        return instance

    class Meta:
        model = Menu
        fields = [
            "name",
            "path",
            "component",
            "redirect",
            "sort",
            "status",
            "meta",
            "parent",
            "children",
            "menu_button",
            "creator",
            "dept_belong_id",
        ]
        extra_kwargs = {"creator": {"write_only": True}, "dept_belong_id": {"write_only": True}}
        read_only_fields = ["id", "children"]


class WebRouterSerializer(CustomModelSerializer):
    """
    前端菜单路由的简单序列化器
    """

    menu_permission = serializers.SerializerMethodField(read_only=True)

    def get_menu_permission(self, instance):
        # 判断是否是超级管理员
        if self.request.user.is_superuser:
            return instance.menuPermission.values_list("value", flat=True)
        else:
            # 根据当前角色获取权限按钮id集合
            permission_ids = self.request.user.role.values_list("permission", flat=True)
            queryset = instance.menuPermission.filter(id__in=permission_ids, menu=instance.id).values_list(
                "value", flat=True
            )
            if queryset:
                return queryset
            else:
                return None

    class Meta:
        model = Menu
        fields = ("id", "parent", "name", "path", "component", "redirect", "sort", "status", "meta", "menu_permission")
        read_only_fields = ["id"]


# 递归获取菜单
def get_child_menu(childs, menu_ids, tree_ids, is_admin=False):
    """
    :param 当前节点:
    :return [{"id": child.id, "title": child.title, "children": []}]:
    """
    children = []
    if childs:
        for child in childs:
            if child.id in menu_ids and child.id not in tree_ids:
                menu_button_list = list(MenuButton.objects.filter(menu=child).values("id", "name", "value"))
                data = {
                    "id": child.id,
                    "path": child.path,
                    "name": child.name,
                    "component": child.component,
                    "meta": child.meta,
                    "label": child.meta.get("title", "") if child.meta else "",
                    "menu_button_list": menu_button_list,
                    "parent_id": child.parent.id,
                    "parent_name": child.parent.name,
                }
                tree_ids.append(child.id)
                if is_admin:
                    _childs = Menu.objects.filter(parent=child).order_by("sort")
                else:
                    _childs = Menu.objects.filter(parent=child, status=True).order_by("sort")
                if _childs:
                    data["children"] = get_child_menu(_childs, menu_ids, tree_ids, is_admin)
                children.append(data)
    return children


class MenuViewSet(CustomModelViewSet):
    """
    菜单管理接口
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = Menu.objects.all()
    serializer_class = MenuSerializer
    create_serializer_class = MenuCreateSerializer
    update_serializer_class = MenuCreateSerializer
    # 模糊查询 包含
    search_fields = ["meta__title"]
    # 精确查询 等于
    filter_fields = ["parent", "name", "status"]

    # extra_filter_backends = []
    @action(methods=["GET"], detail=False, permission_classes=[])
    def web_router(self, request):
        """用于前端获取当前角色的路由"""
        user = request.user
        tree, tree_ids, menus_result = [], [], []
        menu_ids = (
            Menu.objects.values_list("id", flat=True)
            if user.is_superuser
            else user.role.values_list("menu__id", flat=True)
        )
        menus_result = Menu.objects.filter(id__in=menu_ids, status=True, parent=None).order_by("sort")
        for menu in menus_result:
            if menu.id not in tree_ids:
                menu_button_list = list(MenuButton.objects.filter(menu=menu).values("id", "name", "value"))
                menu_data = {
                    "id": menu.id,
                    "path": menu.path,
                    "name": menu.name,
                    "component": menu.component,
                    "meta": menu.meta,
                    "label": menu.meta.get("title", "") if menu.meta else "",
                    "menu_button_list": menu_button_list,
                    "parent_id": menu.parent.id if menu.parent else None,
                    "parent_name": menu.parent.name if menu.parent else None,
                }
                tree_ids.append(menu.id)
                childs = Menu.objects.filter(parent=menu, status=True).order_by("sort")
                if childs:
                    menu_data["children"] = get_child_menu(childs, menu_ids, tree_ids)
                tree.append(menu_data)
        if len(tree) == 0:
            return ErrorResponse(
                code=-1,
                msg="您暂无登录系统权限",
            )
        return SuccessResponse(data=tree, total=len(tree))

    def list(self, request):
        """懒加载"""
        params = request.query_params
        parent = params.get("parent", None)
        if params:
            if parent:
                queryset = self.queryset.filter(parent=parent)
            else:
                queryset = self.queryset
        else:
            queryset = self.queryset.filter(parent__isnull=True)
        queryset = self.filter_queryset(queryset)
        serializer = MenuSerializer(queryset, many=True, request=request)
        data = serializer.data
        return SuccessResponse(data=data)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import logging

from django.contrib.auth.models import AnonymousUser
from django_restql.mixins import DynamicFieldsMixin
from rest_framework import serializers
from rest_framework.fields import empty
from rest_framework.request import Request
from rest_framework.serializers import ModelSerializer

from django_base_ai.system.models import Dept, Users

logger = logging.getLogger(__name__)


class CustomModelSerializer(DynamicFieldsMixin, ModelSerializer):
    """
    增强DRF的ModelSerializer,可自动更新模型的审计字段记录
    (1)self.request能获取到rest_framework.request.Request对象
    """

    # 修改人的审计字段名称, 默认modifier, 继承使用时可自定义覆盖
    modifier_field_id = "modifier"
    modifier_name = serializers.SerializerMethodField(read_only=True)

    def get_modifier_name(self, instance):
        if not hasattr(instance, "modifier"):
            return None
        queryset = Users.objects.filter(id=instance.modifier).values_list("name", flat=True).first()

        if queryset:
            return queryset
        return None

    # 创建人的审计字段名称, 默认creator, 继承使用时可自定义覆盖
    creator_field_id = "creator"
    creator_name = serializers.SerializerMethodField(read_only=True)

    def get_creator_name(self, instance):
        if not hasattr(instance, "creator"):
            return None
        queryset = Users.objects.filter(id=instance.creator).values_list("name", flat=True).first()

        if queryset:
            return queryset
        return None

    avatar_field_id = "avatar"
    creator_avatar = serializers.SerializerMethodField(read_only=True)

    def get_creator_avatar(self, instance):
        if not hasattr(instance, "avatar"):
            return None
        queryset = Users.objects.filter(id=instance.creator).values_list("avatar", flat=True).first()

        if queryset:
            return queryset
        return None

    dept_belong_id = "dept_belong_id"
    dept_info = serializers.SerializerMethodField(read_only=True)

    def get_dept_info(self, instance):
        if not hasattr(instance, "dept_belong_id"):
            return None
        queryset = (
            Dept.objects.filter(id=instance.dept_belong_id).values("id", "name", "short_name", "dept_full_path").first()
        )

        if queryset:
            return queryset
        return None

    # 数据所属部门字段
    dept_belong_id_field_name = "dept_belong_id"
    # 添加默认时间返回格式
    create_datetime = serializers.DateTimeField(format="%Y-%m-%d %H:%M:%S", required=False, read_only=True)
    update_datetime = serializers.DateTimeField(format="%Y-%m-%d %H:%M:%S", required=False)

    def __init__(self, instance=None, data=empty, request=None, **kwargs):
        super().__init__(instance, data, **kwargs)
        self.request: Request = request or self.context.get("request", None)

    def save(self, **kwargs):
        return super().save(**kwargs)

    def create(self, validated_data):
        if self.request and self.request.user and not isinstance(self.request.user, AnonymousUser):
            if self.modifier_field_id in self.fields.fields:
                validated_data[self.modifier_field_id] = self.get_request_user_id()
            if self.creator_field_id in self.fields.fields:
                validated_data[self.creator_field_id] = self.get_request_user_id()

            if (
                self.dept_belong_id_field_name in self.fields.fields
                and validated_data.get(self.dept_belong_id_field_name, None) is None
            ):
                validated_data[self.dept_belong_id_field_name] = getattr(self.request.user, "dept_id", None)
        return super().create(validated_data)

    def update(self, instance, validated_data):
        if self.request and self.request.user and not isinstance(self.request.user, AnonymousUser):
            if self.modifier_field_id in self.fields.fields:
                validated_data[self.modifier_field_id] = self.get_request_user_id()
            if hasattr(self.instance, self.modifier_field_id):
                setattr(self.instance, self.modifier_field_id, self.get_request_user_id())
        return super().update(instance, validated_data)

    def get_request_username(self):
        if getattr(self.request, "user", None):
            user = self.request.user
            # 如果用户是 AnonymousUser，返回 None 而不是 "AnonymousUser"
            if isinstance(user, AnonymousUser):
                return None
            return getattr(user, "username", None)
        return None

    def get_request_name(self):
        if getattr(self.request, "user", None):
            return getattr(self.request.user, "name", None)
        return None

    def get_request_user_id(self):
        if getattr(self.request, "user", None):
            return getattr(self.request.user, "id", None)
        return None

    @property
    def errors(self):
        # get errors
        errors = super().errors
        verbose_errors = {}

        # fields = { field.name: field.verbose_name } for each field in model
        fields = {
            field.name: field.verbose_name
            for field in self.Meta.model._meta.get_fields()
            if hasattr(field, "verbose_name")
        }

        # iterate over errors and replace error key with verbose name if exists
        for field_name, error in errors.items():
            if field_name in fields:
                verbose_errors[str(fields[field_name])] = error
            else:
                verbose_errors[field_name] = error
        return verbose_errors



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : im_chat_question.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 热门问题
import asyncio
import datetime
import json
import logging

import django_filters
from django.http import StreamingHttpResponse
from rest_framework import serializers
from rest_framework.decorators import action
from rest_framework.permissions import AllowAny, IsAuthenticated

from django_base_ai.system.models import AIDeskAppManage, IMChatQA, IMChatQuestion, IMChatQuestionGroup
from django_base_ai.utils.json_request import ExecuteJsonRequest
from django_base_ai.utils.json_response import DetailResponse, ErrorResponse
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class IMChatQuestionSerializer(CustomModelSerializer):
    sub_title = serializers.SerializerMethodField(read_only=True)

    def get_sub_title(self, obj):
        result_sub_title = []
        im_chat_question = IMChatQuestion.objects.filter(parent=obj)
        for item in im_chat_question:
            result_sub_title.append({"nid": item.id, "title": item.title, "is_enable": item.is_enable, "hit": item.hit})
        return result_sub_title

    class Meta:
        model = IMChatQuestion
        fields = "__all__"
        read_only_fields = ["id"]


class IMChatQuestionCreateUpdateSerializer(CustomModelSerializer):
    def save(self, **kwargs):
        instance = super().save(**kwargs)
        sub_title = self.initial_data.get("sub_title", [])
        for item in sub_title:
            IMChatQuestion.objects.update_or_create(
                defaults={
                    "im_chat_question_group": instance.im_chat_question_group,
                    "parent": instance,
                    "title": item.get("title", ""),
                },
                id=item.get("nid", None),
            )
        return instance

    class Meta:
        model = IMChatQuestion
        fields = "__all__"
        read_only_fields = ["id"]


class FilterIMChatQuestion(django_filters.rest_framework.FilterSet):
    im_chat_question_group = django_filters.CharFilter(field_name="im_chat_question_group")
    title = django_filters.CharFilter(field_name="title", lookup_expr="icontains")
    content = django_filters.CharFilter(field_name="content", lookup_expr="icontains")
    is_enable = django_filters.BooleanFilter(field_name="is_enable")

    class Meta:
        model = IMChatQuestion
        fields = ["im_chat_question_group", "title", "content", "is_enable"]


async def event_stream(chats):
    await asyncio.sleep(2)
    yield f"data: {json.dumps(chats)}\n\n"


class IMChatQuestionViewSet(CustomModelViewSet):
    """
    员工服务的热门问题
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = IMChatQuestion.objects.all().filter(parent=None).order_by("hit")
    serializer_class = IMChatQuestionSerializer
    filter_class = FilterIMChatQuestion
    create_serializer_class = IMChatQuestionCreateUpdateSerializer
    update_serializer_class = IMChatQuestionCreateUpdateSerializer
    # extra_filter_backends = []

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def top_question(self, request, *args, **kwargs):
        nid = request.GET.get("nid", 0)
        if not nid:
            return ErrorResponse(msg="参数错误")
        question_group_result_ids = IMChatQuestionGroup.objects.filter(ai_desk_app_manage__id=nid).values_list(
            "id", flat=True
        )
        result = IMChatQuestion.objects.filter(
            im_chat_question_group__in=question_group_result_ids, parent=None
        ).values("id", "title", "content")[0:8]
        return DetailResponse(data=result, msg="获取成功")

    # QA 问答
    @action(methods=["POST"], detail=False, permission_classes=[IsAuthenticated])
    def qa_chat(self, request, *args, **kwargs):
        # 1.热门问题 QA
        # 2.AI
        # 3.人工
        current_user = request.user
        text = request.data.get("text", None)
        nid = request.data.get("nid", None)
        if not text and not nid:
            return ErrorResponse(msg="参数错误")
        ai_desk_app_manage = AIDeskAppManage.objects.filter(id=nid).first()
        if not ai_desk_app_manage:
            return ErrorResponse(msg="不存在或您无权限访问")
        im_chat_question = IMChatQuestion.objects.filter(title__contains=text).first()
        chats = {}  # 默认使用热门问题
        # 获取父级
        im_chat_question = im_chat_question.parent if im_chat_question and im_chat_question.parent else im_chat_question
        if im_chat_question:
            im_chat_question.hit += 1
            im_chat_question.save()
            chats.update({"q_type": 0, "answer": im_chat_question.content})
        elif ai_desk_app_manage.ai_switch:
            # 解析调用ai的json字符串--需要更具不同的AI接口返回进行解析，例如返回 msg,text,content等 还需要注意EventSource
            send_request = ExecuteJsonRequest()
            flag, resp_result, status_code, run_tiem = send_request.request(ai_desk_app_manage.ai_settings_json)
            print(flag, resp_result, status_code, run_tiem)
            answer = json.loads(resp_result).get("msg", "") if flag else "发生错误请稍后再试"
            chats.update({"q_type": 1, "answer": answer})
        else:
            # 未知
            answer = "您的问题我们已收到,我们会尽快回复您"
            chats.update({"q_type": 2, "answer": answer})
        # 统一记录表Q A
        im_chat_qa = IMChatQA.objects.create(
            ai_desk_app_manage=ai_desk_app_manage,
            question=text,
            answer=chats.get("answer", ""),
            q_type=chats.get("q_type", 0),
            creator=current_user.id,
            modifier=current_user.id,
        )
        chats.update({"nid": im_chat_qa.id, "is_end": True})
        return StreamingHttpResponse(event_stream(chats), content_type="text/event-stream")

    @action(methods=["GET", "POST"], detail=False, permission_classes=[AllowAny], authentication_classes=[])
    def ai_test(self, request, *args, **kwargs):
        """
        AI 接口挡板测试
        """
        print(request.data)
        print(request.query_params)
        return DetailResponse(msg=f"AI答案:{datetime.datetime.now()}")



#!/usr/bin/env python
"""
@Project ：django_base_ai
@File    ：client.py
@Author  ：congxing.wang
@Date    ：2026/02/25
@Desc    ：消息客户端与通知发送
"""

import json
import logging

import requests

from django_base_ai import dispatch

logger = logging.getLogger(__name__)


class Client:
    def __init__(self):
        self.domain = dispatch.get_system_config_values("message.client_server_url")
        self.agent_id = dispatch.get_system_config_values("message.client_server_appid")
        self.app_secret = dispatch.get_system_config_values("message.client_server_secret")
        logger.debug("消息客户端初始化 domain=%s agent_id=%s", self.domain, self.agent_id)
        self.get_suite_token = f"{self.domain}/base/api/system/apps/get_suite_token/"
        self.open_api_create_message = f"{self.domain}/base/api/system/message_center/open_api_create_message/"
        self.open_api_get_message = f"{self.domain}/base/api/system/message_center/open_api_get_message/?agent_id="
        self.open_api_message_user_list = (
            f"{self.domain}/base/api/system/message_center_target_user/open_api_message_user_list/?nid="
        )
        try:
            r = requests.request(
                "POST", self.get_suite_token, data={"agent_id": self.agent_id, "secret": self.app_secret}
            )
            token = r.json().get("data", None) if r.ok else None
        except Exception as e:
            logger.warning("获取 suite token 失败: %s", e)
            token = None
        self.headers = {
            "Content-Type": "application/json",
            "Authorization": token,
        }
        logger.debug("消息客户端 headers 已设置")

    def read(self):
        response = requests.request("GET", f"{self.open_api_get_message}{self.agent_id}", headers=self.headers)
        logger.debug("消息列表响应 status=%s", response.status_code)
        return response.json()

    def send(self, payload):
        response = requests.request(
            "POST", f"{self.open_api_create_message}", headers=self.headers, data=json.dumps(payload)
        )
        logger.info("发送消息 status=%s", response.status_code)
        return response.json()

    def user_list(self, nid):
        response = requests.request("GET", f"{self.open_api_get_message}{nid}", headers=self.headers)
        logger.debug("消息用户列表 nid=%s status=%s", nid, response.status_code)
        return response.json()


if __name__ == "__main__":
    import os
    import django
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "DjangoBaseAi.settings")
    django.setup()
    # 需在系统配置中设置 message.client_server_url / message.client_server_appid / message.client_server_secret
    client = Client()
    payload = {
        "title": "通知标题",
        "short_content": "示例内容",
        "message_type": 0,
        "thumbnail": "#",
        "content": "内容",
        "url": "#",
        "send_user_info": [],
    }
    print(client.send(payload))



"""消息中心目标用户 MessageCenterTargetUserViewSet 接口单元测试。"""

import pytest
from rest_framework import status

from tests.assertions import assert_unauthorized

pytestmark = [pytest.mark.django_db]

API = "/base/api/system/message_center_target_user"


def test_message_center_target_user_list_requires_auth(api_client):
    """GET /message_center_target_user/ 未认证返回 401。"""
    response = api_client.get(API + "/")
    assert_unauthorized(response)


def test_message_center_target_user_list_ok(authenticated_client):
    """GET /message_center_target_user/ 已认证返回 200。"""
    response = authenticated_client.get(API + "/")
    assert response.status_code == status.HTTP_200_OK
    assert response.json().get("code") == 2000
